// Copyright (c) 2016 Douglas Miller

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.sound.sampled.*;
import java.io.*;

class BeepDialog extends JPanel implements ActionListener {
	private JComboBox<String> beeps;
	private JButton tryit;
	private JTextField volume;
	JOptionPane jop;
	JDialog dlg;

	public BeepDialog(String[] bps, String bn, String bv) {
		JPanel pn;

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		beeps = new JComboBox<String>(bps);
		for (String f : bps) {
			if (f.equals(bn)) {
				beeps.setSelectedItem(f);
				break;
			}
		}
		add(beeps);

		volume = new JTextField();
		volume.setPreferredSize(new Dimension(50, 20));
		volume.setText(bv);
		pn = new JPanel();
		pn.add(new JLabel("Volume (%):"));
		pn.add(volume);
		add(pn);

		tryit = new JButton("Try It");
		tryit.addActionListener(this);
		pn = new JPanel(); // for some reason, this gives better layout.
		pn.add(tryit);
		add(pn);
		jop = new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION);
	}

	private void tryIt() {
		Clip audio;
		String s = this.volume.getText();
		if (s.isEmpty()) {
			return;
		}
		try {
			AudioInputStream wav =
				AudioSystem.getAudioInputStream(
					new BufferedInputStream(
						this.getClass().
						getResourceAsStream(
							(String)beeps.getSelectedItem())));
			AudioFormat format = wav.getFormat();
			DataLine.Info info = new DataLine.Info(Clip.class, format);
			audio = (Clip)AudioSystem.getLine(info);
			audio.open(wav);
		} catch (Exception e) {
			//e.printStackTrace();
			return;
		}
		int volume = 50;
		if (s != null) {
			volume = Integer.valueOf(s);
			if (volume < 0) volume = 0;
			if (volume > 100) volume = 100;
		}
		FloatControl vol = null;
		if (audio.isControlSupported(FloatControl.Type.MASTER_GAIN)) {
			vol = (FloatControl)audio.getControl(FloatControl.Type.MASTER_GAIN);
		} else if (audio.isControlSupported(FloatControl.Type.VOLUME)) {
			vol = (FloatControl)audio.getControl(FloatControl.Type.VOLUME);
		}
		if (vol != null) {
			float min = vol.getMinimum();
			float max = vol.getMaximum();
			float gain = (float)(min + ((max - min) * (volume / 100.0)));
			vol.setValue(gain);
		}
		audio.setFramePosition(0);
		audio.loop(0);
	}

	public boolean doDialog(Component top, String title) {
		dlg = jop.createDialog(top, title);
		dlg.setVisible(true);
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return false;
		}
		return true;
	}

	public String getBeepName() {
		return (String)beeps.getSelectedItem();
	}

	public String getBeepVolume() {
		return (String)volume.getText();
	}

	public void actionPerformed(ActionEvent e) {
		tryIt();
	}
}
