// Copyright (c) 2016 Douglas Miller

import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class CPNETDialog extends JPanel implements ConfigListener {
	private JTextField cid;
	private JTextField port;
	private Map<Integer, String> nodes;
	private Map<Integer, NetworkNode> targs;
	private String[] srvs = new String[] {
		"HostFileBdos",
		"Socket",
	};

	public CPNETDialog(Properties props) {
		super();
		String s;
		JPanel pn;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		s = props.getProperty("cpnetdevice_clientid");
		cid = new JTextField();
		cid.setPreferredSize(new Dimension(50, 20));
		if (s != null) {
			cid.setText(s);
		}

		s = props.getProperty("cpnetdevice_port");
		port = new JTextField();
		port.setPreferredSize(new Dimension(100, 20));
		if (s != null) {
			port.setText(s);
		}

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Client ID"));
		pn.add(cid);
		add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("I/O Port"));
		pn.add(port);
		add(pn);

		nodes = new HashMap<Integer, String>();
		targs = new HashMap<Integer, NetworkNode>();
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("cpnetdevice_server")) {
				int node = Integer.valueOf(prop.substring(18), 16);
				s = props.getProperty(prop);
				String[] t = s.split("\\s", 2);
				nodes.put(node, t[0]);
				NetworkNode targ = setupTarget(node, t[0], s);
				if (targ != null) {
					targs.put(node, targ);
					targ.setProperties(props, node);
				}
			}
		}
	}

	private NetworkNode setupTarget(int node, String type, String full) {
		// must not update any maps until user clicks OK...
		NetworkNode targ = null;
		if (type.equalsIgnoreCase("HostFileBdos")) {
			targ = new HFBDialog(full);
		} else if (type.equalsIgnoreCase("Socket")) {
			targ = new SocketDialog(full);
		} else {
		}
		return targ;
	}

	public void updateProperties(Properties props) {
		// Must remove all existing servers and rebuild list.
		removeProperties(props);
		String s = cid.getText();
		if (s.isEmpty()) {
			props.remove("cpnetdevice_clientid");
		} else {
			props.setProperty("cpnetdevice_clientid", s);
		}
		s = port.getText();
		if (s.isEmpty()) {
			props.remove("cpnetdevice_port");
		} else {
			props.setProperty("cpnetdevice_port", s);
		}
		for (Integer node : nodes.keySet()) {
			s = String.format("cpnetdevice_server%02x", node);
			props.setProperty(s, nodes.get(node));
			if (targs.containsKey(node)) {
				targs.get(node).updateProperties(props, node);
			}
		}
	}

	public void removeProperties(Properties props) {
		// Removing targets requires that 'nodes' et al has not been updated yet.
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("cpnetdevice_")) {
				props.remove(prop);
			}
			if (prop.startsWith("cpnetdevice_server")) {
				int node = Integer.valueOf(prop.substring(18), 16);
				targs.get(node).removeProperties(props, node);
			}
		}
	}

	public boolean doDialog(Component top, String title) {
		DiskDialog dd = new DiskDialog(this, 0, 255, srvs, "%02x", 200,
			"ID", "Server", this, nodes);
		if (!dd.doDialog(top, "Configure CPNETDevice")) {
			return false;
		}
		nodes = dd.getDriveMap();
		return true;
	}

	public boolean configEnabled(String type) {
		return !type.equals("None");
	}

	public void configChanged(String num, String type) {
		int node = Integer.valueOf(num, 16);
		if (targs.containsKey(node) && !targs.get(node).isType(type)) {
			targs.remove(node);
		}
	}

	public void configPerformed(String num, String type, JTextField fld) {
		int node = Integer.valueOf(num, 16);
		NetworkNode targ = null;
		if (targs.containsKey(node)) {
			targ = targs.get(node);
		} else {
			targ = setupTarget(node, type, null);
		}
		if (targ == null || !targ.configure(this, node)) {
			return;
		}
		// Must save this target, even though user may CANCEL node later...
		// Target should be ignored if node does not exist...
		if (!targs.containsKey(node)) {
			targs.put(node, targ);
		}
	}
}
