// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Map;
import java.util.HashMap;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Properties;

public class ConfigServer implements ActionListener {

	private static ConfigServer thus;

	private JFrame frame;
	private JMenuItem save;

	private Properties props;
	private File curFile = null;
	private File curDir;
	private boolean changed = false;
	static final String title = "Configure Virtual CP/NET Server";

	private JTextField sidField;
	private JTextField rootField;
	private JTextField tempField;
	private JTextField maxField;
	private JTextField hostField;
	private JTextField portField;
	private JButton addb;
	private JButton rootBrowse;

	private File root;
	private Character temp;
	private String sid;
	private String host;
	private String port;
	private String max;
	private Map<Integer, String> lsts;
	private Vector<JTextField> lstField;
	private Vector<JComboBox> typeField;
	private Vector<JTextField> optsField;
	private Vector<String> typeList;
	private int curY;
	private GridBagLayout gb;
	private GridBagConstraints gc;

	private ConfigServer(String[] args) {
		curDir = new File(System.getProperty("user.dir"));
		lsts = new HashMap<Integer, String>();
		lstField = new Vector<JTextField>();
		typeField = new Vector<JComboBox>();
		optsField = new Vector<JTextField>();
		typeList = new Vector<String>();
		typeList.add("None");
		typeList.add(">");
		typeList.add("|");
		typeList.add("Diablo630Stream");

		gb = new GridBagLayout();
		gc = new GridBagConstraints();
		gc.fill = GridBagConstraints.NONE;
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 1;
		gc.weighty = 1;
		gc.gridwidth = 1;
		gc.gridheight = 1;
		gc.insets.left = 2;
		gc.insets.right = 2;
		gc.insets.top = 2;
		gc.insets.bottom = 2;
		gc.anchor = GridBagConstraints.CENTER;

		frame = new JFrame(title);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLayout(gb);

		////////////////////////////////////////////////
		// Setup menus
		//
		JMenuBar mb = new JMenuBar();
		JMenu mu;
		JMenuItem mi;
		mu = new JMenu("File");
		mi = new JMenuItem("New", KeyEvent.VK_N);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Open", KeyEvent.VK_O);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Save", KeyEvent.VK_S);
		save = mi;
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Save As", KeyEvent.VK_A);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Quit", KeyEvent.VK_Q);
		mi.addActionListener(this);
		mu.add(mi);
		mb.add(mu);
		frame.setJMenuBar(mb);

		JPanel pn;
		addb = new JButton("+");
		addb.addActionListener(this);

		sidField = new JTextField();
		sidField.setPreferredSize(new Dimension(100, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Server ID"));
		pn.add(sidField);
		place(pn, 0, 0, 3, 1, GridBagConstraints.CENTER);

		rootBrowse = new JButton("Browse");
		rootBrowse.addActionListener(this);
		rootField = new JTextField();
		rootField.setPreferredSize(new Dimension(200, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Root Dir"));
		pn.add(rootField);
		pn.add(rootBrowse);
		place(pn, 0, 1, 3, 1, GridBagConstraints.CENTER);

		tempField = new JTextField();
		tempField.setPreferredSize(new Dimension(30, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Temp Drive"));
		pn.add(tempField);
		place(pn, 0, 2, 3, 1, GridBagConstraints.CENTER);

		maxField = new JTextField();
		maxField.setPreferredSize(new Dimension(40, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Max Clients"));
		pn.add(maxField);
		place(pn, 0, 3, 3, 1, GridBagConstraints.CENTER);

		hostField = new JTextField();
		hostField.setPreferredSize(new Dimension(200, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Host Address"));
		pn.add(hostField);
		place(pn, 0, 4, 3, 1, GridBagConstraints.CENTER);

		portField = new JTextField();
		portField.setPreferredSize(new Dimension(100, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Host Port"));
		pn.add(portField);
		place(pn, 0, 5, 3, 1, GridBagConstraints.CENTER);
		curY = 6;
		place(addb, 0, curY, 1, 1, GridBagConstraints.CENTER);

		Properties p = new Properties();
		if (args.length > 0) {
			curFile = new File(args[0]);
			try {
				FileInputStream f = new FileInputStream(curFile);
				p.load(f);
			} catch(Exception ee) {
				System.err.format("No config file: %s\n", args[0]);
				System.exit(1);
			}
		}
		setProperties(p);
		changed = false;

		frame.pack();
		frame.setVisible(true);
	}

	private void place(JComponent pn, int x, int y, int dx, int dy, int anc) {
		gc.gridx = x;
		gc.gridy = y;
		gc.gridwidth = dx;
		gc.gridheight = dy;
		gc.anchor = anc;
		gb.setConstraints(pn, gc);
		frame.add(pn);
	}

	private void addHeaders(String lst, String typ, String opt) {
		place(new JLabel(lst), 0, curY, 1, 1, GridBagConstraints.EAST);
		place(new JLabel(typ), 1, curY, 1, 1, GridBagConstraints.CENTER);
		place(new JLabel(opt), 2, curY, 1, 1, GridBagConstraints.WEST);
		++curY;
	}

	private void addList(Integer lst, String typ, String opt) {
		JTextField tf1 = new JTextField();
		tf1.setPreferredSize(new Dimension(30, 20));
		if (lst != null) {
			tf1.setText(String.format("%x", lst));
		}
		JComboBox<String> cb2 = new JComboBox<String>(typeList);
		if (typ != null) {
			for (String t : typeList) {
				if (t.equals(typ)) {
					cb2.setSelectedItem(t);
					break;
				}
			}
		}
		JTextField tf3 = new JTextField();
		tf3.setPreferredSize(new Dimension(100, 20));
		if (opt != null) {
			tf3.setText(opt);
		}
		lstField.add(tf1);
		typeField.add(cb2);
		optsField.add(tf3);
		place(tf1, 0, curY, 1, 1, GridBagConstraints.EAST);
		place(cb2, 1, curY, 1, 1, GridBagConstraints.CENTER);
		place(tf3, 2, curY, 1, 1, GridBagConstraints.WEST);
		++curY;
	}

	// TODO: options for generating commandline args?
	private void setProperties(Properties p) {
		if (curFile == null) {
			save.setEnabled(false);
			frame.setTitle(title + " - No File");
		} else {
			save.setEnabled(true);
			frame.setTitle(title + " - " + curFile.getName());
		}
		props = p;
		String s;
		// cpnetserver_root_dir
		// cpnetserver_temp
		// cpnetserver_sid
		// cpnetserver_host
		// cpnetserver_port
		// cpnetserver_max
		// cpnetserver_lst#
		root = null;
		s = props.getProperty("cpnetserver_root_dir");
		if (s != null) {
			root = new File(s);
		}
		temp = null;
		s = props.getProperty("cpnetserver_temp");
		if (s != null && s.matches("[a-pA-P]:?")) {
			temp = Character.toUpperCase(s.charAt(0));
		}
		sid = null;
		s = props.getProperty("cpnetserver_sid");
		if (s != null) {
			sid = s;
		}
		host = null;
		s = props.getProperty("cpnetserver_host");
		if (s != null) {
			host = s;
		}
		port = null;
		s = props.getProperty("cpnetserver_port");
		if (s != null) {
			port = s;
		}
		max = null;
		s = props.getProperty("cpnetserver_port");
		if (s != null) {
			max = s;
		}
		lsts.clear();
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("cpnetserver_lst")) {
				int lst = Integer.valueOf(prop.substring(15), 16);
				s = props.getProperty(prop);
				lsts.put(lst, s);
			}
		}
	}

	private void updateProperties() {
	}

	private boolean confirmChanges(String op) {
		if (changed) {
			int res = confirm(frame, op, "Changes have not been saved. " +
					"Discard changes?");
			if (res == JOptionPane.YES_OPTION) {
				return true;
			}
			return false;
		}
		return true;
	}

	private void saveFile(boolean pick) {
		updateProperties();
		if (pick || curFile == null) {
			File f = pickFile(frame, curFile == null ? curDir : curFile,
						"Save File");
			if (f == null) {
				return;
			}
			curFile = f;
			frame.setTitle(title + " - " + curFile.getName());
		}
		try {
			FileOutputStream f = new FileOutputStream(curFile);
			props.store(f, "Updated by ConfigServer");
			changed = false;
		} catch (Exception ee) {
			error(frame, "Save File", ee.getMessage());
		}
	}

	private void setupFile(File fi) {
		Properties p = new Properties();
		// TEMP during bring-up...
		if (fi != null) {
			try {
				FileInputStream f = new FileInputStream(fi);
				p.load(f);
				curFile = fi;
			} catch (Exception ee) {
				error(frame, "Open File", ee.getMessage());
			}
		}
		setProperties(p);
		changed = false;
	}

	private File pickFile(Component top, File cur, String purpose) {
		File file = cur;
		SuffFileChooser ch = new SuffFileChooser(purpose,
			null, null, null, null, cur == null ? curDir : cur, 0);
		int rv = ch.showDialog(top);
		if (rv == JFileChooser.APPROVE_OPTION) {
			file = ch.getSelectedFile();
		}
		// still could be null?
		return file;
	}

	private void menuClick(JMenuItem mi) {
		if (mi.getMnemonic() == KeyEvent.VK_N) {
			if (!confirmChanges("New File")) {
				return;
			}
			curFile = null;
			setProperties(new Properties());
			changed = false;
		} else if (mi.getMnemonic() == KeyEvent.VK_O) {
			if (!confirmChanges("Open File")) {
				return;
			}
			setupFile(pickFile(frame, curFile, "Open File"));
		} else if (mi.getMnemonic() == KeyEvent.VK_S) {
			saveFile(false);
		} else if (mi.getMnemonic() == KeyEvent.VK_A) {
			saveFile(true);
		} else if (mi.getMnemonic() == KeyEvent.VK_Q) {
			if (!confirmChanges("Quit")) {
				return;
			}
			System.exit(0);
		}
	}

	private int getNext() {
		long m = 0;
		for (int x = 0; x < lstField.size(); ++x) {
			String s = lstField.get(x).getText();
			if (s.isEmpty()) {
				continue;
			}
			int i = Integer.valueOf(s, 16);
			if (i < 0 || i > 15) {
				continue;
			}
			m |= (1 << i);
		}
		for (int x = 0; x < 16; ++x) {
			if ((m & (1 << x)) == 0) {
				return x;
			}
		}
		return -1;
	}


	private void newList() {
		int n = getNext();
		Integer idx = null;
		if (n >= 0) {
			idx = n;
		}
		frame.remove(addb);
		addList(idx, null, null);
		place(addb, 0, curY, 1, 1, GridBagConstraints.CENTER);
		frame.validate();
		frame.pack();
		frame.repaint();
	}

	private void pickDir() {
		String s = rootField.getText();
		File f = null;
		if (!s.isEmpty()) {
			f = new File(s);
		}
		SuffFileChooser sfc = new SuffFileChooser("Root CP/NET Dir",
			null, null, null, null, f, 0);
		sfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int rv = sfc.showDialog(frame);
		if (rv == JFileChooser.APPROVE_OPTION) {
			f = sfc.getSelectedFile();
			rootField.setText(f.getAbsolutePath());
			root = f;
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof JMenuItem) {
			menuClick((JMenuItem)e.getSource());
		} else if (e.getSource() == rootBrowse) {
			pickDir();
		} else if (e.getSource() == addb) {
			newList();
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}

	static public void info(Component top, String op, String err) {
		JOptionPane.showMessageDialog(top,
			new JLabel(err),
			op, JOptionPane.INFORMATION_MESSAGE);
	}

	static public void error(Component top, String op, String err) {
		JOptionPane.showMessageDialog(top,
			new JLabel(err),
			op + " Error", JOptionPane.ERROR_MESSAGE);
	}

	static public int confirm(Component top, String op, String err) {
		int res = JOptionPane.showConfirmDialog(top,
			new JLabel(err),
			op + " Confirmation", JOptionPane.YES_NO_OPTION);
		return res;
	}

	public static void main(String[] args) {
		thus = new ConfigServer(args);
	}
}
