// Copyright (c) 2016 Douglas Miller

import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class Diablo630Dialog extends JPanel implements Peripheral {
	private JComboBox<String> cpiField;
	private JComboBox<String> lpiField;
	private JTextField paperField;
	private JComboBox<String> fontName;
	private JComboBox<String> fontStyle;
	private JTextField fontSize;
	private JComboBox<String> paperType;
	private JComboBox<String> orient;
	private JComboBox<String> jobAction;
	private JTextField jobField;
	private JCheckBox nogui;
	private JCheckBox greenbar;
	String[] argv = null; // one-shot after ctor only...

	String[] cpis = new String[]{
		"Default",
		"10",
		"12",
		"15",
	};
	String[] lpis = new String[]{
		"Default",
		"6",
		"8",
	};
	String[] jobs = new String[]{
		"Default",
		"NONE",
		"DISCARD",
		"SAVE",
		"QUEUE",
	};
	String[] fontList;
	String[] stys = new String[]{
		"Default",
		"PLAIN",
		"BOLD",
		"ITALIC",
		"BOLD ITALIC",
	};
	String[] paps = new String[]{
		"Default",
		"LETTER",
		"LEGAL",
		"FORMS",
	};
	String[] oris = new String[]{
		"Default",
		"PORTRAIT",
		"LANDSCAPE",
	};

	public Diablo630Dialog(String args) {
		super();
		JPanel pn;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		if (args != null) {
			argv = args.split("\\s");
		}

		String[] fl = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
		fontList = new String[fl.length + 1];
		fontList[0] = "Default";
		for (int x = 0; x < fl.length; ++x) {
			fontList[x + 1] = fl[x];
		}

		cpiField = new JComboBox<String>(cpis);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Chars/Inch: "));
		pn.add(cpiField);
		add(pn);

		lpiField = new JComboBox<String>(lpis);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Lines/Inch: "));
		pn.add(lpiField);
		add(pn);

		nogui = new JCheckBox("No Operator Console");
		add(nogui);

		JSeparator sp = new JSeparator();
		sp.setPreferredSize(new Dimension(50, 20)); // X should not matter
		add(sp);

		fontName = new JComboBox<String>(fontList);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Font Name: "));
		pn.add(fontName);
		add(pn);
		fontStyle = new JComboBox<String>(stys);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Font Style: "));
		pn.add(fontStyle);
		add(pn);
		fontSize = new JTextField();
		fontSize.setPreferredSize(new Dimension(50, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Font Size: "));
		pn.add(fontSize);
		add(pn);

		sp = new JSeparator();
		sp.setPreferredSize(new Dimension(50, 20)); // X should not matter
		add(sp);

		paperType = new JComboBox<String>(paps);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Paper: "));
		pn.add(paperType);
		add(pn);
		orient = new JComboBox<String>(oris);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Orientation: "));
		pn.add(orient);
		add(pn);
		greenbar = new JCheckBox("Greenbar");
		add(greenbar);

		sp = new JSeparator();
		sp.setPreferredSize(new Dimension(50, 20)); // X should not matter
		add(sp);

		jobAction = new JComboBox<String>(jobs);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Job End Action: "));
		pn.add(jobAction);
		add(pn);
		jobField = new JTextField();
		jobField.setPreferredSize(new Dimension(200, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Job End Args: "));
		pn.add(jobField);
		add(pn);
	}

	public boolean isType(String type) {
		return type.startsWith("Diablo630");
	}

	private void setSelect(String s, String[] vals, JComboBox fld) {
		for (String t : vals) {
			if (t.equals(s)) {
				fld.setSelectedItem(t);
				return;
			}
		}
		fld.setSelectedItem(vals[0]);
	}

	private void setFont(String s, String sep) {
		String[] t = s.split(sep);
		if (t.length > 0) {
			setSelect(t[0].replace('_', ' '), fontList, fontName);
		}
		if (t.length > 1) {
			setSelect(t[1], stys, fontStyle);
		}
		if (t.length > 2) {
			fontSize.setText(t[2]);
		}
	}

	private void setPaper(String s, String sep) {
		String[] t = s.split(sep);
		if (t.length > 0) {
			setSelect(t[0], paps, paperType);
		}
		if (t.length > 1) {
			setSelect(t[1], oris, orient);
		}
		greenbar.setSelected(t.length > 2 && t[2].equalsIgnoreCase("greenbar"));
	}

	private void setJob(String s, String sep) {
		String[] t = s.split(sep, 2);
		if (t.length > 0) {
			setSelect(t[0], jobs, jobAction);
		}
		if (t.length > 1) {
			jobField.setText(t[1].replace(';', ' '));
		}
	}

	private void setupArgv(String[] argv, int num) {
		for (String s : argv) {
			if (s.startsWith("cpi=")) {
				setSelect(s.substring(4), cpis, cpiField);
				continue;
			}
			if (s.startsWith("lpi=")) {
				setSelect(s.substring(4), lpis, lpiField);
				continue;
			}
			if (s.equals("nogui")) {
				nogui.setSelected(true);
				continue;
			}
			if (s.startsWith("font=")) {
				setFont(s.substring(5), ",");
				continue;
			}
			if (s.startsWith("paper=")) {
				setPaper(s.substring(6), ",");
				continue;
			}
			if (s.startsWith("endjob=")) {
				setJob(s.substring(7), ",");
				continue;
			}
		}
	}

	public void setArgs(String args, int num) {
		if (argv != null) {
			setupArgv(argv, num);
			argv = null;
		}
		setupArgv(args.split("\\s"), num);
	}

	public String getArgs(int num) {
		String ret = "";
		String s = (String)cpiField.getSelectedItem();
		if (!s.equals("Default")) {
			ret += " cpi=" + s;
		}
		s = (String)lpiField.getSelectedItem();
		if (!s.equals("Default")) {
			ret += " lpi=" + s;
		}
		if (nogui.isSelected()) {
			ret += " nogui";
		}
		boolean pfx = false;
		s = (String)fontName.getSelectedItem();
		if (!s.equals("Default")) {
			ret += " font=" + s.replace(' ', '_');
			pfx = true;
		}
		s = (String)fontStyle.getSelectedItem();
		if (!s.equals("Default")) {
			if (!pfx) {
				ret += " font=";
				pfx = true;
			}
			ret += ',';
			ret += s;
		} else if (pfx) {
			ret += ',';
		}
		s = fontSize.getText();
		if (!s.isEmpty()) {
			if (!pfx) {
				ret += " font=,";
				pfx = true;
			}
			ret += ',';
			ret += s;
		} else if (pfx) {
			ret += ',';
		}
		pfx = false;
		s = (String)paperType.getSelectedItem();
		if (!s.equals("Default")) {
			ret += " paper=" + s;
			pfx = true;
		}
		s = (String)orient.getSelectedItem();
		if (!s.equals("Default")) {
			if (!pfx) {
				ret += " paper=";
				pfx = true;
			}
			ret += ',';
			ret += s;
		} else if (pfx) {
			ret += ',';
		}
		if (greenbar.isSelected()) {
			if (!pfx) {
				ret += " paper=,";
				pfx = true;
			}
			ret += ',';
			ret += "GREENBAR";
		} else if (pfx) {
			ret += ',';
		}
		s = (String)jobAction.getSelectedItem();
		if (!s.equals("Default")) {
			ret += " endjob=" + s;
			s = jobField.getText();
			if (!s.isEmpty()) {
				ret += "," + s.replace(' ', ';');
			}
		}
		// first char is blank, remove it
		if (!ret.isEmpty()) {
			ret = ret.substring(1);
		}
		return ret;
	}

	public void setProperties(Properties props, int num) {
		String s;
		s = props.getProperty("diablo630_cpi");
		setSelect(s, cpis, cpiField);
		s = props.getProperty("diablo630_lpi");
		setSelect(s, lpis, lpiField);
		nogui.setSelected(props.getProperty("diablo630_nogui") != null);
		s = props.getProperty("diablo630_font");
		if (s != null) {
			setFont(s, "\\s");
		}
		s = props.getProperty("diablo630_paper");
		if (s != null) {
			setPaper(s, "\\s");
		}
		s = props.getProperty("diablo630_jobend");
		if (s != null) {
			setJob(s, "\\s");
		}
	}

	public void updateProperties(Properties props, int num) {
		String s;
		if (argv != null) {
			setupArgv(argv, num);
			argv = null;
		}
		s = (String)cpiField.getSelectedItem();
		if (!s.equals("Default")) {
			props.setProperty("diablo630_cpi", s);
		} else {
			props.remove("diablo630_cpi");
		}
		s = (String)lpiField.getSelectedItem();
		if (!s.equals("Default")) {
			props.setProperty("diablo630_lpi", s);
		} else {
			props.remove("diablo630_lpi");
		}
		if (nogui.isSelected()) {
			props.setProperty("diablo630_nogui", "true");
		} else {
			props.remove("diablo630_nogui");
		}
		String t = "";
		s = (String)fontName.getSelectedItem();
		if (!s.equals("Default")) {
			t += s.replace(' ', '_');
		}
		s = (String)fontStyle.getSelectedItem();
		if (!s.equals("Default")) {
			if (t.isEmpty()) {
				t += '.';
			}
			t += ' ';
			t += s;
		}
		s = fontSize.getText();
		if (!s.isEmpty()) {
			if (t.isEmpty()) {
				t += ". ";
			}
			t += ' ';
			t += s;
		}
		if (!t.isEmpty()) {
			props.setProperty("diablo630_font", t);
		} else {
			props.remove("diablo630_font");
		}
		t = "";
		s = (String)paperType.getSelectedItem();
		if (!s.equals("Default")) {
			t += s;
		}
		s = (String)orient.getSelectedItem();
		if (!s.equals("Default")) {
			if (t.isEmpty()) {
				t += '.';
			}
			t += ' ';
			t += s;
		}
		if (greenbar.isSelected()) {
			if (t.isEmpty()) {
				t += ". ";
			}
			t += ' ';
			t += "GREENBAR";
		}
		if (!t.isEmpty()) {
			props.setProperty("diablo630_paper", t);
		} else {
			props.remove("diablo630_paper");
		}
		t = "";
		s = (String)jobAction.getSelectedItem();
		if (!s.equals("Default")) {
			t += s;
			s = jobField.getText();
			if (!s.isEmpty()) {
				t += ' ' + s;
			}
		}
		if (!t.isEmpty()) {
			props.setProperty("diablo630_jobend", t);
		} else {
			props.remove("diablo630_jobend");
		}
	}

	public void removeProperties(Properties props, int num) {
		String pfx = "diablo630_";
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith(pfx)) {
				props.remove(prop);
			}
		}
	}

	public boolean configure(Component top, int num) {
		if (argv != null) {
			setupArgv(argv, num);
			argv = null;
		}
		JOptionPane jop = new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
					JOptionPane.OK_CANCEL_OPTION);

		JDialog dlg = jop.createDialog(top, "Diablo 630 Power-on Settings");
		dlg.setVisible(true);
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return false;
		}
		return true;
	}
}
