// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Vector;
import javax.swing.*;
import java.util.Properties;

// Used for any generic Map<Integer, String> set of configs...

public class DiskDialog extends JPanel implements ActionListener {
	static ImageIcon cfgBtn = new ImageIcon(DiskDialog.class.getResource("icons/config.png"));
	static ImageIcon cfgOffBtn = new ImageIcon(DiskDialog.class.getResource("icons/config_dis.png"));

	private JComboBox<String> types;
	private JButton addb;
	private GridBagLayout gb;
	private GridBagConstraints gc;
	private Vector<String> typeList;
	private Vector<JTextField> indexField;
	private Vector<JComboBox> typeField;
	private Vector<JTextField> optsField = null;
	private Vector<DiskImageFile> media = null;
	private Vector<JTextField> mediField = null;
	private Vector<JButton> brsField = null;
	private Vector<JButton> cfgField = null;
	private ConfigListener optLstn = null;
	private String idxFmt = "%d";
	private int idxBase = 10;
	private int optWidth = 100;
	private JOptionPane jop;
	private JDialog dlg;
	private int first;
	private int num;
	private static String[] sufx = {"logdisk", "mmsdisk" };
	private static String[] sufd = {"Sector Image", "Raw Image" };
	private static File cur = null;

	// For floppy disks (removeable media)...
	public DiskDialog(int first, int num,
			String[] tys, String fmt, int optWidth,
			String col1, String col2, String col3,
			String col4, // mounted media, for removable drives
			Map<Integer, String> drvs,
			Map<Integer, DiskImageFile> dsks) {
		super();
		diskDialog(null, first, num, tys, fmt, optWidth,
			col1, col2, col3, col4,
			drvs, dsks);
	}

	// networks or harddrives... no media to remove...
	public DiskDialog(JPanel panel, int first, int num,
			String[] tys, String fmt, int optWidth,
			String col1, String col2, Object col3,
			Map<Integer, String> drvs) {
		super();
		diskDialog(panel, first, num, tys, fmt, optWidth,
			col1, col2, col3, null,
			drvs, null);
	}

	// network server LST: devices
	public DiskDialog(JPanel panel, int first, int num,
			String[] tys, String fmt, int optWidth,
			String col1, String col2, Object col3, String col4,
			Map<Integer, String> drvs) {
		super();
		diskDialog(panel, first, num, tys, fmt, optWidth,
			col1, col2, col3, col4,
			drvs, null);
	}

	private void diskDialog(JPanel panel, int first, int num,
			String[] tys, String fmt, int optWidth,
			String col1, String col2, Object obj3,
			String col4, // mounted media, for removable drives
			Map<Integer, String> drvs,
			Map<Integer, DiskImageFile> dsks) {
		this.first = first;
		this.num = num;
		if (cur == null) {
			cur = new File(System.getProperty("user.dir"));
		}
		gb = new GridBagLayout();
		gc = new GridBagConstraints();
		gc.fill = GridBagConstraints.NONE;
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 1;
		gc.weighty = 1;
		gc.gridwidth = 1;
		gc.gridheight = 1;
		gc.insets.left = 2;
		gc.insets.right = 2;
		gc.insets.top = 2;
		gc.insets.bottom = 2;
		gc.anchor = GridBagConstraints.CENTER;

		if (fmt != null) {
			idxFmt = fmt;
			idxBase = idxFmt.endsWith("x") ? 16 : 10;
		}
		if (optWidth > this.optWidth) {
			this.optWidth = optWidth;
		}
		if (col1 == null) {
			col1 = "Num";
		}
		if (col2 == null) {
			col2 = "Drive";
		}
		String col3 = null;
		if (obj3 instanceof String) {
			col3 = (String)obj3;
		} else if (obj3 instanceof ConfigListener) {
			optLstn = (ConfigListener)obj3;
			cfgField = new Vector<JButton>();
			if (col4 != null) {
				col3 = col4;
				col4 = null;
			}
		} else {
			col3 = "Label";
		}

		indexField = new Vector<JTextField>();
		typeField = new Vector<JComboBox>();
		if (col3 != null) {
			optsField = new Vector<JTextField>();
		}
		if (dsks != null) {
			media = new Vector<DiskImageFile>();
			mediField = new Vector<JTextField>();
			brsField = new Vector<JButton>();
		}
		typeList = new Vector<String>();
		typeList.add("None");
		for (String s : tys) {
			typeList.add(s);
		}

		setLayout(gb);
		setOpaque(true);
		addb = new JButton("+");
		addb.addActionListener(this);

		// text  combo  text
		// [idx] [type] [opts]
		// ...
		// [+]

		if (panel != null) {
			gc.gridwidth = 3;
			gb.setConstraints(panel, gc);
			add(panel);
			gc.gridwidth = 1;
			gc.gridx = 0;
			++gc.gridy;
		}
		addHeadings(col1, col2, col3, col4);
		for (Integer x : drvs.keySet()) {
			String s = drvs.get(x);
			String[] d;
			if (!Character.isLetterOrDigit(s.charAt(0))) {
				d = new String[]{ "" + s.charAt(0), s.substring(1) };
			} else {
				d = s.split("\\s", 2);;
			}
			String o = null;
			if (d.length > 1) {
				o = d[1];
			}
			DiskImageFile m = null;
			if (dsks != null && dsks.containsKey(x)) {
				m = dsks.get(x);
			}
			addDrive(x, d[0], o, m);
		}

		gb.setConstraints(addb, gc);
		add(addb);
		jop = new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
			JOptionPane.OK_CANCEL_OPTION);
	}

	private void addDrive(Integer idx, String typ, String opt, DiskImageFile med) {
		JTextField tf1 = new JTextField();
		tf1.setPreferredSize(new Dimension(30, 20));
		if (idx != null) {
			tf1.setText(String.format(idxFmt, idx + first));
		}
		JComboBox<String> cb2 = new JComboBox<String>(typeList);
		if (typ != null) {
			for (String t : typeList) {
				if (t.equals(typ)) {
					cb2.setSelectedItem(t);
					break;
				}
			}
		}

		indexField.add(tf1);
		typeField.add(cb2);
		if (optLstn != null) {
			cb2.addActionListener(this);
		}
		gb.setConstraints(tf1, gc);
		add(tf1);
		++gc.gridx;
		gb.setConstraints(cb2, gc);
		add(cb2);
		++gc.gridx;

		if (optsField != null) {
			JTextField tf3 = new JTextField();
			tf3.setPreferredSize(new Dimension(optWidth, 20));
			if (opt != null) {
				tf3.setText(opt);
			}
			gb.setConstraints(tf3, gc);
			add(tf3);
			optsField.add(tf3);
			++gc.gridx;
		}
		if (optLstn != null) {
			JButton cfg = new JButton(cfgBtn);
			cfg.setDisabledIcon(cfgOffBtn);
			cfg.addActionListener(this);
			doCfgEnable(cb2, cfg);
			gb.setConstraints(cfg, gc);
			add(cfg);
			cfgField.add(cfg);
			++gc.gridx;
		}
		if (mediField != null) {
			JTextField tf4 = new JTextField();
			tf4.setPreferredSize(new Dimension(200, 20));
			media.add(med);
			if (med != null) {
				tf4.setText(med.getText());
			}
			mediField.add(tf4);
			gb.setConstraints(tf4, gc);
			add(tf4);
			++gc.gridx;
			JButton brs = new JButton("Browse");
			brs.addActionListener(this);
			gb.setConstraints(brs, gc);
			add(brs);
			brsField.add(brs);
			++gc.gridx;
		}
		gc.gridx = 0;
		++gc.gridy;
	}

	// 'opt' and 'med' may be null...
	// if 'opt' then leave blank, if 'med' then leave off.
	private void addHeadings(String num, String typ, String opt, String med) {
		JLabel lb = new JLabel(num);
		gb.setConstraints(lb, gc);
		add(lb);
		++gc.gridx;
		lb = new JLabel(typ);
		gb.setConstraints(lb, gc);
		add(lb);
		++gc.gridx;
		// if null, leave field empty (but still create space).
		if (opt != null) {
			lb = new JLabel(opt);
			gb.setConstraints(lb, gc);
			add(lb);
		}
		if (mediField != null) {
			++gc.gridx;
			lb = new JLabel(med);
			gb.setConstraints(lb, gc);
			add(lb);
		}
		gc.gridx = 0;
		++gc.gridy;
	}

	public boolean doDialog(Component top, String title) {
		dlg = jop.createDialog(top, title);
		dlg.setVisible(true); 
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return false;
		}
		return true;
	}

	public Map<Integer, String> getDriveMap() {
		Map<Integer, String> ret = new HashMap<Integer, String>();
		for (int x = 0; x < indexField.size(); ++x) {
			String s = indexField.get(x).getText();
			if (s.isEmpty()) {
				continue;
			}
			int i = Integer.valueOf(s, idxBase);
			if (i < first || i >= first + num) {
				continue;
			}
			s = (String)typeField.get(x).getSelectedItem();
			if (s.equals("None")) {
				continue;
			}
			if (optsField != null) {
				if (s.length() > 1 ||
						Character.isLetterOrDigit(s.charAt(0))) {
					s += " ";
				}
				s += optsField.get(x).getText();
			}
			ret.put(i - first, s);
		}
		return ret;
	}

	public Map<Integer, DiskImageFile> getMediaMap() {
		Map<Integer, DiskImageFile> ret = new HashMap<Integer, DiskImageFile>();
		if (mediField == null) {
			return ret;
		}
		for (int x = 0; x < indexField.size(); ++x) {
			String s = indexField.get(x).getText();
			if (s.isEmpty()) {
				continue;
			}
			int i = Integer.valueOf(s, idxBase);
			if (i < first || i >= first + num) {
				continue;
			}
			s = (String)typeField.get(x).getSelectedItem();
			if (s.equals("None")) {
				continue;
			}
			if (media.get(x) != null) {
				ret.put(i - first, media.get(x));
			}
		}
		return ret;
	}

	private void pickImage(int idx) {
		File f = media.get(idx);
		if (f == null) {
			f = cur;
		}
		SuffFileChooser sfc = new SuffFileChooser("Pre-mount image",
			sufx, sufd, null, null, f,
			SuffFileChooser.WRITE_PROTECT | SuffFileChooser.UNMOUNT);
		sfc.setProtected(media.get(idx) == null || media.get(idx).getOpt("rw") < 0);
		int rv = sfc.showDialog(this);
		if (rv == JFileChooser.APPROVE_OPTION) {
			f = sfc.getSelectedFile();
			cur = f;
			// TODO: do we transfer options from previous value?
			DiskImageFile df = new DiskImageFile(f.getAbsolutePath());
			if (!sfc.isProtected()) {
				df.setOpt("rw");
			} else {
				df.delOpt("rw");
			}
			media.set(idx, df);
			mediField.get(idx).setText(df.getText());
		} else if (sfc.isUnmount()) {
			media.set(idx, null);
			mediField.get(idx).setText("");
		}
	}

	private int getNext() {
		long m = 0;
		for (int x = 0; x < indexField.size(); ++x) {
			String s = indexField.get(x).getText();
			if (s.isEmpty()) {
				continue;
			}
			int i = Integer.valueOf(s, idxBase) - first;
			if (i < 0 || i >= num || i > 63) {
				continue;
			}
			m |= (1 << i);
		}
		for (int x = 0; x < num; ++x) {
			if ((m & (1 << x)) == 0) {
				return x;
			}
		}
		return -1;
	}

	private void doCfgEnable(JComboBox cb, JButton bt) {
		String s = (String)cb.getSelectedItem();
		bt.setEnabled(optLstn.configEnabled(s));
	}

	private void doCfgChanged(int x) {
		JComboBox cb = typeField.get(x);
		doCfgEnable(cb, cfgField.get(x));
		optLstn.configChanged(indexField.get(x).getText(),
			(String)cb.getSelectedItem());
	}

	public void actionPerformed(ActionEvent e) {
		if ((e.getSource() instanceof JComboBox)) {
			JComboBox cb = (JComboBox)e.getSource();
			for (int x = 0; x < typeField.size(); ++x) {
				if (cb == typeField.get(x)) {
					doCfgChanged(x);
					return;
				}
			}
			return;
		} else if (!(e.getSource() instanceof JButton)) {
			return;
		}
		JButton bp = (JButton)e.getSource();
		if (bp == addb) {
			int n = getNext();
			Integer idx = null;
			if (n >= 0) {
				idx = n;
			}
			remove(addb);
			addDrive(idx, null, null, null);
			gb.setConstraints(addb, gc);
			add(addb);
			dlg.validate();
			dlg.pack();
			dlg.repaint();
			return;
		}
		for (int x = 0; brsField != null && x < brsField.size(); ++x) {
			if (bp == brsField.get(x)) {
				pickImage(x);
				return;
			}
		}
		for (int x = 0; cfgField != null && x < cfgField.size(); ++x) {
			if (bp == cfgField.get(x)) {
				optLstn.configPerformed(indexField.get(x).getText(),
					(String)typeField.get(x).getSelectedItem(),
					optsField == null ? null : optsField.get(x));
				return;
			}
		}
	}

	public static void setProperties(Properties props) {
		String s = props.getProperty("h89_image_sufx");
		String t = props.getProperty("h89_image_sufd");
		if (s != null && t != null) {
			String[] ss = s.split("[ \t]");
			String[] tt = t.split("[ \t]");
			if (ss.length == tt.length && ss.length > 0) {
				for (int x = 0; x < tt.length; ++x) {
					tt[x] = tt[x].replaceAll("_"," ");
				}
				sufx = ss;
				sufd = tt;
			}
		}
	}
}
