// Copyright (c) 2016 Douglas Miller

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import javax.swing.*;

class FontDialog extends JPanel implements ActionListener {
	private JComboBox<String> fonts;
	private JButton pick;
	private JButton tryit;
	private JTextField size;
	private JTextField color;
	private JTextField tryout;
	private String[] flist;
	JOptionPane jop;
	JDialog dlg;

	public FontDialog(String[] addons, String fn, String fz, String fc) {
		JPanel pn;

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		flist = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
		String[] cbl = new String[addons.length + flist.length];
		System.arraycopy(addons, 0, cbl, 0, addons.length);
		System.arraycopy(flist, 0, cbl, addons.length, flist.length);
		
		fonts = new JComboBox<String>(cbl);
		for (String f : cbl) {
			if (f.equals(fn)) {
				fonts.setSelectedItem(f);
				break;
			}
		}
		add(fonts);

		size = new JTextField();
		size.setPreferredSize(new Dimension(50, 20));
		size.setText(fz);
		pn = new JPanel();
		pn.add(new JLabel("Point Size:"));
		pn.add(size);
		add(pn);

		color = new JTextField();
		color.setPreferredSize(new Dimension(80, 20));
		color.setText(fc);
		pick = new JButton("Pick");
		pick.addActionListener(this);
		pn = new JPanel();
		pn.add(new JLabel("Color (rgb hex):"));
		pn.add(color);
		pn.add(pick);
		add(pn);

		tryit = new JButton("Try It");
		tryit.addActionListener(this);
		pn = new JPanel(); // for some reason, this gets better layout
		pn.add(tryit);
		add(pn);
		tryout = new JTextField(" H: Boot A0> B:*.* ", 30);
		tryout.setOpaque(true);
		tryout.setBackground(new Color(50,50,50));
		setupTryout();
		add(tryout);

		jop = new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION);
	}

	private void setupTryout() {
		int col = 0x00ff00;
		int siz = 20;
		if (!color.getText().isEmpty()) {
			col = Integer.valueOf(color.getText(), 16);
		}
		if (!size.getText().isEmpty()) {
			siz = Integer.valueOf(size.getText());
		}
		tryout.setForeground(new Color(col));
		String fon = (String)fonts.getSelectedItem();
		Font font = null;
		if (fon.endsWith(".ttf")) {
			try {
				File ff = new File(fon);
				java.io.InputStream ttf;
				if (ff.exists()) {
					ttf = new FileInputStream(ff);
				} else {
					ttf = FontDialog.class.getResourceAsStream(fon);
				}
				if (ttf != null) {
					font = Font.createFont(Font.TRUETYPE_FONT, ttf);
					font = font.deriveFont((float)siz);
				}
			} catch (Exception ee) {
				font = null;
			}
		}
		if (font == null) {
			font = new Font(fon, Font.PLAIN, siz);
		}
		// TODO: pop-up error message?
		if (font != null) {
			tryout.setFont(font);
		}
	}

	public boolean doDialog(Component top, String title) {
		dlg = jop.createDialog(top, title);
		dlg.setVisible(true);
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return false;
		}
		return true;
	}

	private void pickColor() {
		int c = 0x00ff00;
		if (!color.getText().isEmpty()) {
			c = Integer.valueOf(color.getText(), 16);
		}
		Color clr = JColorChooser.showDialog(this, "Z19 Screen Color", new Color(c));
		if (clr != null) {
			color.setText(String.format("%06x", clr.getRGB() & 0xffffff));
		}
	}

	public String getFontName() {
		return (String)fonts.getSelectedItem();
	}

	public String getFontSize() {
		return (String)size.getText();
	}

	public String getFontColor() {
		return (String)color.getText();
	}

	public void actionPerformed(ActionEvent e) {
		if (!(e.getSource() instanceof JButton)) {
			return;
		}
		if (e.getSource() == pick) {
			pickColor();
			return;
		}
		setupTryout();
		dlg.validate();
		dlg.pack();
		dlg.repaint();
	}
}
