// Copyright (c) 2016 Douglas Miller

import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class HFBDialog extends JPanel implements NetworkNode, ConfigListener {
	private JTextField dirField;
	private JTextField tmpField;
	private File dir = null;
	private Character temp = null;
	private Map<Integer, String> lsts;
	private String[] srvs = new String[] {
		">",
		"|",
		"Diablo630Stream",
	};
	String[] argv = null; // one-shot after ctor only...

	public HFBDialog(String args) {
		super();
		JPanel pn;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		if (args != null) {
			argv = args.split("\\s");
		}

		dirField = new JTextField();
		dirField.setPreferredSize(new Dimension(200, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Root Dir"));
		pn.add(dirField);
		add(pn);

		// TODO: use combo box?
		tmpField = new JTextField();
		tmpField.setPreferredSize(new Dimension(50, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Temp Drive"));
		pn.add(tmpField);
		add(pn);

		lsts = new HashMap<Integer, String>();
	}

	public boolean isType(String type) {
		return type.equalsIgnoreCase("HostFileBdos");
	}

	// hostfilebdos%02x_root_dir
	// hostfilebdos%02x_temp
	// hostfilebdos%02x_lst%x
	public void setProperties(Properties props, int node) {
		String s;
		String pfx = String.format("hostfilebdos%02x_", node);
		dir = null;
		s = props.getProperty(pfx + "root_dir");
		if (s != null) {
			dir = new File(s);
		}
		temp = null;
		s = props.getProperty(pfx + "temp");
		if (s != null && s.matches("[a-pA-P]:?")) {
			temp = Character.toUpperCase(s.charAt(0));
		}
		if (argv != null) {
			// argv[0] is "HostFileBdos"...
			// TODO: syntax check?
			if (argv.length > 1) {
				if (!argv[1].isEmpty()) {
					dir = new File(argv[1]);
					dirField.setText(argv[1]);
				}
				if (argv.length > 2 && !argv[2].isEmpty()) {
					temp = Character.toUpperCase(argv[2].charAt(0));
					tmpField.setText(argv[2]);
				}
			}
			argv = null;
		}
		pfx += "lst";
		int pflen = pfx.length();
		lsts.clear();
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith(pfx)) {
				int lst = Integer.valueOf(prop.substring(pflen), 16);
				s = props.getProperty(prop);
				lsts.put(lst, s);
			}
		}
	}

	public void updateProperties(Properties props, int node) {
		String pfx = String.format("hostfilebdos%02x_", node);
		// Must remove all existing printers and rebuild list.
		removeProperties(props, node);
		if (dir != null) {
			props.setProperty(pfx + "root_dir", dir.getAbsolutePath());
		}
		if (temp != null) {
			props.setProperty(pfx + "temp", "" + temp);
		}
		for (Integer lst : lsts.keySet()) {
			String s = pfx + String.format("lst%01x", lst);
			props.setProperty(s, lsts.get(lst));
		}
	}

	public void removeProperties(Properties props, int node) {
		String pfx = String.format("hostfilebdos%02x_", node);
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith(pfx)) {
				props.remove(prop);
			}
		}
	}

	public boolean configure(Component top, int node) {
		DiskDialog dd = new DiskDialog(this, 0, 16, srvs, "%01x", 200,
			"LST:", "Action", this, "Args", lsts);
		if (!dd.doDialog(top, String.format("Configure HostFileBdos %02x", node))) {
			return false;
		}
		lsts = dd.getDriveMap();
		String s = dirField.getText();
		if (!s.isEmpty()) {
			dir = new File(s);
		}
		s = tmpField.getText();
		if (s.matches("[a-pA-P]:?")) {
			temp = Character.toUpperCase(s.charAt(0));
		}
		return true;
	}

	private Peripheral setupTarget(int node, String type, String full) {
		// must not update any maps until user clicks OK...
		Peripheral targ = null;
		if (type.startsWith("Diablo630")) {
			targ = new Diablo630Dialog(full);
		} else {
		}
		return targ;
	}


	public boolean configEnabled(String type) {
		return type.startsWith("Diablo630");
	}

	public void configChanged(String num, String type) {
		// no cached context, don't care.
	}

	public void configPerformed(String num, String type, JTextField fld) {
		int lst = Integer.valueOf(num, 16);
		Peripheral targ = setupTarget(lst, type, lsts.get(lst));
		if (targ == null || !targ.configure(this, lst)) {
			return;
		}
		String args = targ.getArgs(lst); // never null?
		if (args != null) {
			fld.setText(args);
		} else {
			fld.setText("");
		}
	}
}
