// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;

public class Jumper2xN extends JPanel implements Jumper, ActionListener {
	private ImageIcon on;
	private ImageIcon off;
	private JButton[] jmprs;
	private int value = 0;
	private ChangeListener lstn;

	// 'n' may be negative (reverse bits in value)
	public Jumper2xN(String label, int n, ChangeListener lstn,
			ImageIcon off, ImageIcon on) {
		super();
		this.lstn = lstn;
		this.off = off;
		this.on = on;
		boolean reverse = (n < 0);
		if (reverse) {
			n = -n;
		}
		jmprs = new JButton[n];
		ImageObserver io = off.getImageObserver();
		Image im = off.getImage();
		int h = im.getHeight(io);
		int w = im.getWidth(io);
		int dx = 0;
		int dy = 1;
		if (w > h) {
			// Must be stacked vertically, each oriented horizontally...
		} else {
			int t = h;
			h = w;
			w = t;
			dx = 1;
			dy = 0;
		}
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gc = new GridBagConstraints();
		setLayout(gb);
		setToolTipText(label);
		//setPreferredSize(new Dimension(w, h * n));
		setOpaque(false);
		gc.fill = GridBagConstraints.NONE;
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 0;
		gc.weighty = 0;
		gc.gridwidth = 1;
		gc.gridheight = 1;
		gc.anchor = GridBagConstraints.CENTER;
		for (int x = 0; x < n; ++x) {
			int xx = x;
			if (reverse) {
				xx = n - x - 1;
			}
			jmprs[xx] = new JButton();
			jmprs[xx].addActionListener(this);
			jmprs[xx].setBorder(null);
			jmprs[xx].setBackground(AddOnCard.PCB);
			jmprs[xx].setToolTipText(label);
			gb.setConstraints(jmprs[xx], gc);
			add(jmprs[xx]);
			gc.gridy += dy;
			gc.gridx += dx;
		}
		setValue(0);
	}

	public void setValue(int val) {
		value = val;
		for (int x = 0; x < jmprs.length; ++x) {
			if ((value & (1 << x)) == 0) {
				jmprs[x].setIcon(off);
			} else {
				jmprs[x].setIcon(on);
			}
		}
		if (lstn != null) {
			lstn.propertyChanged();
		}
		repaint();
	}

	public int getValue() {
		return value;
	}

	public void actionPerformed(ActionEvent e) {
		int v = value;
		for (int x = 0; x < jmprs.length; ++x) {
			if (e.getSource() == jmprs[x]) {
				v ^= (1 << x);
				break;
			}
		}
		setValue(v);
	}
}
