// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Map;
import java.util.HashMap;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Properties;

public class MMS77316 extends AddOnCard
		implements ChangeListener, WindowListener, ActionListener {

	private IDC34 p601;
	private IDC50 p602;

	private Map<Integer, String> drives8;
	private Map<Integer, String> drives5;
	private Map<Integer, DiskImageFile> disks8;
	private Map<Integer, DiskImageFile> disks5;
	private ChangeListener lstn;
	private String[] types8 = new String[]{
		"FDD_8_SS",
		"FDD_8_DS",
	};
	private String[] types5 = new String[]{
		"FDD_5_25_SS_ST",
		"FDD_5_25_DS_ST",
		"FDD_5_25_SS_DT",
		"FDD_5_25_DS_DT",
	};

	public MMS77316(ChangeListener lstn) {
		super("Configure MMS77316");
		this.lstn = lstn;

		drives8 = new HashMap<Integer, String>();
		drives5 = new HashMap<Integer, String>();
		disks8 = new HashMap<Integer, DiskImageFile>();
		disks5 = new HashMap<Integer, DiskImageFile>();

		addWindowListener(this);

		layoutCard();
		connectorListener(this);

		////////////////////////////////////////////////
		// Setup menus
		//
		JMenuBar mb = new JMenuBar();
		JMenu mu;
		JMenuItem mi;
		mu = new JMenu("Card");
		mi = new JMenuItem("Remove", KeyEvent.VK_R);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Done", KeyEvent.VK_D);
		mi.addActionListener(this);
		mu.add(mi);
		// TODO: can we cancel (cleanly) here?
		mb.add(mu);
		setJMenuBar(mb);

		////////////////////////////////////////////////
		// Layout MMS77316 PCB...
		//
		JPanel pn;

		gc.anchor = GridBagConstraints.NORTH;
		p601 = new IDC34("P601", "mms77316_drive");
		p601.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p601);
		pn.add(new JLabel("5\" FLOPPY"));
		place(pn, 1, 0, 3, 1);

		p602 = new IDC50("P602", "mms77316_drive");
		p602.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p602);
		pn.add(new JLabel("8\" FLOPPY"));
		place(pn, 4, 0, 2, 1);

		pn = new JPanel();
		pn.setPreferredSize(new Dimension(840, 350));
		pn.setOpaque(false);
		pn.add(new JLabel("MMS 77316 Double Density Controller"));
		place(pn, 1, 1, 5, 1);
		gc.anchor = GridBagConstraints.CENTER;

		finish();
	}

	public void setProperties(Properties props) {
		String s;
		// TODO: handle mms77316_dir...?
		drives8.clear();
		drives5.clear();
		disks8.clear();
		disks5.clear();
		for (int x = 0; x < 4; ++x) {
			s = String.format("mms77316_drive%d", x + 1);
			s = props.getProperty(s);
			if (s != null) {
				drives8.put(x, s);
			}
			s = String.format("mms77316_disk%d", x + 1);
			s = props.getProperty(s);
			if (s != null) {
				disks8.put(x, new DiskImageFile(s.split("\\s")));
			}
		}
		for (int x = 0; x < 4; ++x) {
			s = String.format("mms77316_drive%d", x + 5);
			s = props.getProperty(s);
			if (s != null) {
				drives5.put(x, s);
			}
			s = String.format("mms77316_disk%d", x + 5);
			s = props.getProperty(s);
			if (s != null) {
				disks5.put(x, new DiskImageFile(s.split("\\s")));
			}
		}
		p601.setValue(drives5.size());
		p602.setValue(drives8.size());
	}

	public void updateProperties(Properties props) {
		String s;
		for (Integer drv : drives5.keySet()) {
			s = String.format("mms77316_drive%d", drv + 5);
			props.setProperty(s, drives5.get(drv));
		}
		for (Integer drv : disks5.keySet()) {
			s = String.format("mms77316_disk%d", drv + 5);
			props.setProperty(s, disks5.get(drv).getProp());
		}
		for (Integer drv : drives8.keySet()) {
			s = String.format("mms77316_drive%d", drv + 1);
			props.setProperty(s, drives8.get(drv));
		}
		for (Integer drv : disks8.keySet()) {
			s = String.format("mms77316_disk%d", drv + 1);
			props.setProperty(s, disks8.get(drv).getProp());
		}
	}

	public void removeProperties(Properties props) {
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("mms77316_")) {
				props.remove(prop);
			}
		}
	}

	public String getCardName() { return "MMS77316"; }

	public boolean configure(Component top) {
		setVisible(true);
		return true;
	}

	public void windowActivated(WindowEvent e) { }
	public void windowClosed(WindowEvent e) { }
	public void windowIconified(WindowEvent e) { }
	public void windowOpened(WindowEvent e) { }
	public void windowDeiconified(WindowEvent e) { }
	public void windowDeactivated(WindowEvent e) { }
	public void windowClosing(WindowEvent e) {
		setVisible(false);
	}

	private void drive5Plug(IDC34 cs) {
		DiskDialog dd = new DiskDialog(5, 4, types5, null, 0,
			"Num", "Drive Type", "Label", "Media", drives5, disks5);
		if (!dd.doDialog(this, "Configure 5\" Drives")) {
			return;
		}
		drives5 = dd.getDriveMap();
		disks5 = dd.getMediaMap();
		p601.setValue(drives5.size());
		propertyChanged();
	}

	private void drive8Plug(IDC50 cs) {
		DiskDialog dd = new DiskDialog(1, 4, types8, null, 0,
			"Num", "Drive Type", "Label", "Media", drives8, disks8);
		if (!dd.doDialog(this, "Configure 8\" Drives")) {
			return;
		}
		drives8 = dd.getDriveMap();
		disks8 = dd.getMediaMap();
		p602.setValue(drives8.size());
		propertyChanged();
	}

	private void menuClick(JMenuItem mi) {
		if (mi.getMnemonic() == KeyEvent.VK_R) {
			setVisible(false);
			if (lstn != null) {
				lstn.cardRemoved(this);
			}
		} else if (mi.getMnemonic() == KeyEvent.VK_D) {
			setVisible(false);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (connectorPerformed(e)) {
			setVisible(false);
		} else if (e.getSource() instanceof IDC34) {
			drive5Plug((IDC34)e.getSource());
		} else if (e.getSource() instanceof IDC50) {
			drive8Plug((IDC50)e.getSource());
		} else if (e.getSource() instanceof JMenuItem) {
			menuClick((JMenuItem)e.getSource());
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}

	public void cardRemoved(AddOnCard cd) {} // never called from widgets

	public void propertyChanged() {
		// TODO: what else?
		if (lstn != null) {
			lstn.propertyChanged();
		}
	}
}
