// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Map;
import java.util.HashMap;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Properties;

public class MMS77320 extends AddOnCard
		implements ChangeListener, WindowListener, ActionListener {

	private Dipsw8 sw1;
	private Jumper jp1;
	private Jumper jp2;
	private IDC40 p603;
	private SerialPorts sports;
	private File dir = null;

	private JTextField dirField;
	private JButton dirBrowse;
	private JPanel dirPanel;

	private Map<Integer, String> drives;
	private ChangeListener lstn;
	private String[] ctrls = new String[]{
		"XEBEC_ST506",
		"XEBEC_ST412",
		"XEBEC_CM5206",
		"XEBEC_CM5410",
		"XEBEC_CM5616",
		"XEBEC_RO201",
		"XEBEC_RO202",
		"XEBEC_RO203",
		"XEBEC_RO204",
	};

	public MMS77320(ChangeListener lstn) {
		super("Configure MMS77320");
		this.lstn = lstn;
		sports = new SerialPorts("P603", "P604", "P605", this, this);

		drives = new HashMap<Integer, String>();

		addWindowListener(this);

		layoutCard();
		connectorListener(this);

		////////////////////////////////////////////////
		// Setup menus
		//
		JMenuBar mb = new JMenuBar();
		JMenu mu;
		JMenuItem mi;
		mu = new JMenu("Card");
		mi = new JMenuItem("Remove", KeyEvent.VK_R);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Done", KeyEvent.VK_D);
		mi.addActionListener(this);
		mu.add(mi);
		// TODO: can we cancel (cleanly) here?
		mb.add(mu);
		setJMenuBar(mb);

		dirField = new JTextField();
		if (dir != null) {
			dirField.setText(dir.getAbsolutePath());
		}
		dirField.setPreferredSize(new Dimension(200, 20));
		dirBrowse = new JButton("Browse");
		dirBrowse.addActionListener(this);
		dirPanel = new JPanel();
		dirPanel.setLayout(new BoxLayout(dirPanel, BoxLayout.X_AXIS));
		dirPanel.add(new JLabel("Dir:"));
		dirPanel.add(dirField);
		dirPanel.add(dirBrowse);

		////////////////////////////////////////////////
		// Layout MMS77320 PCB...
		//
		JPanel pn;
		JPanel pn2;

		pn2 = new JPanel();
		pn2.setLayout(new BoxLayout(pn2, BoxLayout.X_AXIS));
		pn2.setOpaque(false);

		p603 = new IDC40("P601", "mms77320_drive");
		p603.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p603);
		pn.add(new JLabel("SASI"));
		pn2.add(pn);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(10,10));
		pn.setOpaque(false);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(sports.lpCon);
		pn.add(new JLabel("PRINTER"));
		pn2.add(pn);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(10,10));
		pn.setOpaque(false);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(sports.auxCon);
		pn.add(new JLabel("AUX"));
		pn2.add(pn);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(10,10));
		pn.setOpaque(false);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(sports.modemCon);
		pn.add(new JLabel("MODEM"));
		pn2.add(pn);

		gc.anchor = GridBagConstraints.NORTH;
		place(pn2, 1, 0, 5, 1);

		pn = new JPanel();
		pn.setPreferredSize(new Dimension(840, 300));
		pn.setOpaque(false);
		pn.add(new JLabel("MMS 77320 SASI Interface w/Serial Ports"));
		place(pn, 1, 1, 5, 1);

		gc.anchor = GridBagConstraints.SOUTH;
		gc.gridwidth = 1;
		gc.gridheight = 1;
		sw1 = new Dipsw0("SW1", this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(sw1);
		pn.add(new JLabel("SW1"));
		place(pn, 2, 2, 1, 1);

		Jumper3Vert jp;
		Jumper2xNHoriz j3;

		jp1 = jp = new Jumper3Vert("JP1", this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("78"));
		pn.add(jp);
		pn.add(new JLabel("7C"));
		place(pn, 3, 2, 1, 1);

		pn2 = new JPanel();
		pn2.setLayout(new BoxLayout(pn2, BoxLayout.X_AXIS));
		pn2.setOpaque(false);

		jp2 = j3 = new Jumper2xNHoriz("JP2", 3, this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("SASI"));
		pn.add(j3);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("LP"));
		pn.add(sports.lpJp);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("AUX"));
		pn.add(sports.auxJp);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("MDM"));
		pn.add(sports.modemJp);
		pn2.add(pn);

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel(" "));
		pn.add(new JLabel("INT3"));
		pn.add(new JLabel("INT4"));
		pn.add(new JLabel("INT5"));
		pn2.add(pn);
		gc.anchor = GridBagConstraints.SOUTHWEST;
		place(pn2, 4, 2, 2, 1);

		gc.anchor = GridBagConstraints.CENTER;
		finish();
	}

	public void setProperties(Properties props) {
		sports.setProperties(props);
		int v = 0;
		String s = props.getProperty("mms77320_dipsw");
		if (s != null) {
			v = Integer.valueOf(s, 2) & 0xff;
		}
		sw1.setValue(v);
		s = props.getProperty("mms77320_port");
		v = 0;
		if (s != null && s.equalsIgnoreCase("jp1b")) {
			v |= 1;
		}
		jp1.setValue(v);
		v = 0;
		s = props.getProperty("mms77320_intr");
		if (s != null) {
			int i = 0;
			if (s.startsWith("jp2")) {
				i = s.charAt(3) - 'a' + 3;
			} else {
				i = Integer.valueOf(s);
			}
			if (i == 3) {
				v |= 1;
			} else if (i == 4) {
				v |= 2;
			} else if (i == 5) {
				v |= 4;
			}
		}
		jp2.setValue(v);
		// TODO: handle mms77320_dir...
		// fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		drives.clear();
		for (int x = 0; x < 8; ++x) {
			s = String.format("mms77320_drive%d", x);
			s = props.getProperty(s);
			if (s != null) {
				drives.put(x, s);
			}
		}
		p603.setValue(drives.size());
		s = props.getProperty("mms77320_dir");
		// must differentiate 'null' from cwd.
		// 'null' means we don't want to choose dir...
		if (s != null) {
			dir = new File(s);
		} else {
			dir = null;
		}
	}

	public void updateProperties(Properties props) {
		sports.updateProperties(props);
		String s;
		props.setProperty("mms77320_dipsw", sw1.getProp());
		int v = jp1.getValue();
		if (v == 0) {
			props.setProperty("mms77320_port", "0x78");
		} else {
			props.setProperty("mms77320_port", "0x7c");
		}
		v = jp2.getValue();
		if ((v & 4) != 0) {
			props.setProperty("mms77320_intr", "5");
		} else if ((v & 2) != 0) {
			props.setProperty("mms77320_intr", "4");
		} else if ((v & 1) != 0) {
			props.setProperty("mms77320_intr", "3");
		}
		for (Integer drv : drives.keySet()) {
			s = String.format("mms77320_drive%d", drv);
			props.setProperty(s, drives.get(drv));
		}
		if (dir != null) {
			props.setProperty("mms77320_dir", dir.getAbsolutePath());
		} else {
			props.remove("mms77320_dir");
		}
	}

	public void removeProperties(Properties props) {
		sports.removeProperties(props);
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("mms77320_")) {
				props.remove(prop);
			}
		}
	}

	public String getCardName() { return "MMS77320"; }

	public boolean configure(Component top) {
		setVisible(true);
		return true;
	}

	public void windowActivated(WindowEvent e) { }
	public void windowClosed(WindowEvent e) { }
	public void windowIconified(WindowEvent e) { }
	public void windowOpened(WindowEvent e) { }
	public void windowDeiconified(WindowEvent e) { }
	public void windowDeactivated(WindowEvent e) { }
	public void windowClosing(WindowEvent e) {
		setVisible(false);
	}

	private void drivePlug(IDC40 cs) {
		DiskDialog dd = new DiskDialog(dirPanel, 0, 8, ctrls, null, 0,
			"Num", "Ctrl/Drv", "Label", drives);
		if (!dd.doDialog(this, "Configure SASI Controllers/Drives")) {
			return;
		}
		if (dirField.getText().isEmpty()) {
			dir = null;
		} else {
			dir = new File(dirField.getText());
		}
		drives = dd.getDriveMap();
		p603.setValue(drives.size());
		propertyChanged();
	}

	private void menuClick(JMenuItem mi) {
		if (mi.getMnemonic() == KeyEvent.VK_R) {
			setVisible(false);
			if (lstn != null) {
				lstn.cardRemoved(this);
			}
		} else if (mi.getMnemonic() == KeyEvent.VK_D) {
			setVisible(false);
		}
	}

	private void pickDir() {
		String s = dirField.getText();
		File f = null;
		if (!s.isEmpty()) {
			f = new File(s);
		}
		SuffFileChooser sfc = new SuffFileChooser("Image Dir",
			null, null, null, null, f, 0);
		sfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int rv = sfc.showDialog(this);
		if (rv == JFileChooser.APPROVE_OPTION) {
			f = sfc.getSelectedFile();
			dirField.setText(f.getAbsolutePath());
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == dirBrowse) {
			pickDir();
		} else if (connectorPerformed(e)) {
			setVisible(false);
		} else if (e.getSource() instanceof IDC40) {
			drivePlug((IDC40)e.getSource());
		} else if (e.getSource() instanceof JMenuItem) {
			menuClick((JMenuItem)e.getSource());
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}

	public void cardRemoved(AddOnCard cd) {} // never called from widgets

	public void propertyChanged() {
		// TODO: what else?
		if (lstn != null) {
			lstn.propertyChanged();
		}
	}
}
