// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Map;
import java.util.HashMap;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Properties;

public class MMS77422 extends AddOnCard
		implements ChangeListener, ConfigListener,
			WindowListener, ActionListener {
	private Jumper3x1 port;
	private Jumper2xN intr;
	private Jumper2xN nodeId;
	private IDC20 p604;
	private Serial15 lpCon;

	private Map<Integer, String> nodes;
	private Map<Integer, NetworkNode> targs;
	private ChangeListener lstn;

	private String[] srvs = new String[] {
		"HostFileBdos",
		"Socket",
	};

	public MMS77422(ChangeListener lstn) {
		super("Configure MMS77422");
		this.lstn = lstn;

		nodes = new HashMap<Integer, String>();
		targs = new HashMap<Integer, NetworkNode>();

		addWindowListener(this);

		layoutCard();
		connectorListener(this);

		////////////////////////////////////////////////
		// Setup menus
		//
		JMenuBar mb = new JMenuBar();
		JMenu mu;
		JMenuItem mi;
		mu = new JMenu("Card");
		mi = new JMenuItem("Remove", KeyEvent.VK_R);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Done", KeyEvent.VK_D);
		mi.addActionListener(this);
		mu.add(mi);
		// TODO: can we cancel (cleanly) here?
		mb.add(mu);
		setJMenuBar(mb);

		////////////////////////////////////////////////
		// Layout MMS77422 PCB...
		//
		JPanel pn;

		gc.anchor = GridBagConstraints.NORTH;
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		lpCon = new Serial15("P603", "mms77422_lp_att");
		lpCon.addActionListener(this);
		pn.add(lpCon);
		pn.add(new JLabel("PRINTER"));
		place(pn, 2, 0, 2, 1);

		p604 = new IDC20("P604", "mms77422_server");
		p604.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p604);
		pn.add(new JLabel("NETWORK"));
		place(pn, 4, 0, 1, 1);
		gc.anchor = GridBagConstraints.CENTER;

		nodeId = new Jumper2xNVert("NTWKID", -6, this);
		pn = new JPanel();
		pn.setOpaque(false);
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.add(new JLabel("NTWK ID"));
		pn.add(nodeId);
		place(pn, 2, 1, 1, 1);

		pn = new JPanel();
		pn.setPreferredSize(new Dimension(270,300));
		pn.setOpaque(false);
		pn.add(new JLabel("MMS 77422 Network Interface"));
		place(pn, 3, 1, 2, 1);

		gc.anchor = GridBagConstraints.SOUTH;
		port = new Jumper3Horiz("PORT", this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("78 7C"));
		pn.add(port);
		place(pn, 3, 2, 1, 1);

		intr = new Jumper2xNVert("INT", 3, this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("3 4 5 INT"));
		pn.add(intr);
		place(pn, 4, 2, 1, 1);
		gc.anchor = GridBagConstraints.CENTER;

		finish();
	}

	public void setProperties(Properties props) {
		String s = props.getProperty("mms77422_port");
		int v = 1;
		if (s != null) {
			int p = Integer.decode(s);
			if (p == 0x7c) {
				v = 0;
			}
		}
		port.setValue(v);
		s = props.getProperty("mms77422_intr");
		v = 4;
		if (s != null) {
			int p = Integer.valueOf(s);
			if (p == 3) {
				v = 1;
			} else if (p == 4) {
				v = 2;
			} else if (p == 5) {
				v = 4;
			}
		}
		intr.setValue(v);
		s = props.getProperty("mms77422_clientid");
		v = 0;
		if (s != null) {
			int p = Integer.decode(s);
			if (p >= 0 && p <= 63) {
				v = p;
			}
		}
		nodeId.setValue(v);
		nodes.clear();
		targs.clear();
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("mms77422_server")) {
				int node = Integer.valueOf(prop.substring(15), 16);
				s = props.getProperty(prop);
				String[] t = s.split("\\s", 2);
				nodes.put(node, t[0]);
				NetworkNode targ = setupTarget(node, t[0], s);
				if (targ != null) {
					targs.put(node, targ);
					targ.setProperties(props, node);
				}
			}
		}
		p604.setValue(nodes.size());
	}

	private NetworkNode setupTarget(int node, String type, String full) {
		// must not update any maps until user clicks OK...
		NetworkNode targ = null;
		if (type.equalsIgnoreCase("HostFileBdos")) {
			targ = new HFBDialog(full);
		} else if (type.equalsIgnoreCase("Socket")) {
			targ = new SocketDialog(full);
		} else {
		}
		return targ;
	}

	public void updateProperties(Properties props) {
		String s;
		int v = port.getValue();
		if (v == 0) {
			s = "0x7c";
		} else {
			s = "0x78";
		}
		props.setProperty("mms77422_port", s);
		v = intr.getValue();
		if ((v & 4) != 0) {
			props.setProperty("mms77422_intr", "5");
		} else if ((v & 2) != 0) {
			props.setProperty("mms77422_intr", "4");
		} else if ((v & 1) != 0) {
			props.setProperty("mms77422_intr", "3");
		} else {
			props.remove("mms77422_intr");
		}
		v = nodeId.getValue();
		props.setProperty("mms77422_clientid", Integer.toString(v));
		// Must remove all existing servers, since current list
		// does not include ones that were deleted.
		// TODO: if other code detects deletes and removes props,
		// this is not needed.
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("mms77422_server")) {
				props.remove(prop);
			}
		}
		for (Integer node : nodes.keySet()) {
			s = String.format("mms77422_server%02x", node);
			props.setProperty(s, nodes.get(node));
			if (targs.containsKey(node)) {
				targs.get(node).updateProperties(props, node);
			}
		}
	}

	public void removeProperties(Properties props) {
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("mms77422_")) {
				props.remove(prop);
			}
			if (prop.startsWith("mms77422_server")) {
				int node = Integer.valueOf(prop.substring(15), 16);
				targs.get(node).removeProperties(props, node);
			}
		}
	}

	public String getCardName() { return "MMS77422"; }

	public boolean configure(Component top) {
		setVisible(true);
		return true;
	}

	public void windowActivated(WindowEvent e) { }
	public void windowClosed(WindowEvent e) { }
	public void windowIconified(WindowEvent e) { }
	public void windowOpened(WindowEvent e) { }
	public void windowDeiconified(WindowEvent e) { }
	public void windowDeactivated(WindowEvent e) { }
	public void windowClosing(WindowEvent e) {
		setVisible(false);
	}

	private void netPlug(IDC20 cs) {
		// re-purpose diskDialog for network servers...
		DiskDialog dd = new DiskDialog(null, 0, 255, srvs, "%02x", 200,
			"ID", "Server", this, nodes);
		if (!dd.doDialog(this, "Configure Network Nodes")) {
			return;
		}
		nodes = dd.getDriveMap();
		p604.setValue(nodes.size());
		propertyChanged();
	}

	private void lpPlug(Serial15 cs) {
		ConfigH89.info(this, "Config Printer", "Configuration of 77422 printer not yet supported");
		// propertyChanged();
	}

	private void menuClick(JMenuItem mi) {
		if (mi.getMnemonic() == KeyEvent.VK_R) {
			setVisible(false);
			if (lstn != null) {
				lstn.cardRemoved(this);
			}
		} else if (mi.getMnemonic() == KeyEvent.VK_D) {
			setVisible(false);
		}
	}

	public boolean configEnabled(String type) {
		return !type.equals("None");
	}

	public void configChanged(String num, String type) {
		int node = Integer.valueOf(num, 16);
		if (targs.containsKey(node) && !targs.get(node).isType(type)) {
			targs.remove(node);
		}
	}

	public void configPerformed(String num, String type, JTextField fld) {
		int node = Integer.valueOf(num, 16);
		NetworkNode targ = null;
		if (targs.containsKey(node)) {
			targ = targs.get(node);
		} else {
			targ = setupTarget(node, type, null);
		}
		if (targ == null || !targ.configure(this, node)) {
			return;
		}
		// Must save this target, even though user may CANCEL node later...
		// Target should be ignored if node does not exist...
		if (!targs.containsKey(node)) {
			targs.put(node, targ);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof IDC20) {
			netPlug((IDC20)e.getSource());
		} else if (connectorPerformed(e)) {
			setVisible(false);
		} else if (e.getSource() instanceof Serial15) {
			lpPlug((Serial15)e.getSource());
		} else if (e.getSource() instanceof JMenuItem) {
			menuClick((JMenuItem)e.getSource());
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}

	public void cardRemoved(AddOnCard cd) {} // never called from widgets

	public void propertyChanged() {
		// TODO: what else?
		if (lstn != null) {
			lstn.propertyChanged();
		}
	}
}
