// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.awt.*;
import java.awt.event.*;
import java.util.Properties;
import javax.swing.*;

public class SerialPorts implements ActionListener {
	static ImageIcon cfgBtn = new ImageIcon(SerialPorts.class.getResource("icons/config.png"));
	static ImageIcon cfgOffBtn = new ImageIcon(SerialPorts.class.getResource("icons/config_dis.png"));

	public Jumper2xNHoriz lpJp;
	public Jumper2xNHoriz auxJp;
	public Jumper2xNHoriz modemJp;
	public Serial15 lpCon;
	public Serial15 auxCon;
	public Serial15 modemCon;
	private ChangeListener lstn;
	private Component top;
	private String[] srvs = new String[] {
		"None",
		">",
		"!",
		"Diablo630Serial",
	};

	public SerialPorts(String lpLab, String auxLab, String modemLab,
			ChangeListener lstn, Component top) {
		this.lstn = lstn;
		this.top = top;
		if (lpLab != null) {
			lpCon = new Serial15(lpLab, "lp_att");
			lpCon.addActionListener(this);
			lpJp = new Jumper2xNHoriz("JP3", 3, lstn);
		}
		if (auxLab != null) {
			auxCon = new Serial15(auxLab, "aux_att");
			auxCon.addActionListener(this);
			auxJp = new Jumper2xNHoriz("JP4", 3, lstn);
		}
		if (modemLab != null) {
			modemCon = new Serial15(modemLab, "modem_att");
			modemCon.addActionListener(this);
			modemJp = new Jumper2xNHoriz("JP5", 3, lstn);
		}
	}

	private void setPort(Properties props, Jumper jp, Serial15 pt, String pfx) {
		if (pt == null) {
			return;
		}
		int v = 0;
		String s = props.getProperty(pfx + "intr");
		if (s != null) {
			int i = Integer.valueOf(s);
			if (i == 3) {
				v |= 1;
			} else if (i == 4) {
				v |= 2;
			} else if (i == 5) {
				v |= 4;
			}
		}
		jp.setValue(v);
		s = props.getProperty(pfx + "att");
		pt.setValue(s);
	}

	private void getPort(Properties props, Jumper jp, Serial15 pt, String pfx) {
		if (pt == null) {
			return;
		}
		String prop = pfx + "intr";
		int v = jp.getValue();
		String i = null;
		if ((v & 1) != 0) {
			i = "3";
		} else if ((v & 2) != 0) {
			i = "4";
		} else if ((v & 4) != 0) {
			i = "5";
		}
		if (i != null) {
			props.setProperty(prop, i);
		} else {
			props.remove(prop);
		}
		prop = pfx + "att";
		i = pt.getValue();
		if (i != null) {
			props.setProperty(prop, i);
		} else {
			props.remove(prop);
		}
	}

	public void setProperties(Properties props) {
		setPort(props, lpJp, lpCon, "lp_");
		setPort(props, auxJp, auxCon, "aux_");
		setPort(props, modemJp, modemCon, "modem_");
	}

	public void updateProperties(Properties props) {
		getPort(props, lpJp, lpCon, "lp_");
		getPort(props, auxJp, auxCon, "aux_");
		getPort(props, modemJp, modemCon, "modem_");
	}

	public void removeProperties(Properties props) {
		for (String prop : props.stringPropertyNames()) {
			if ((lpCon != null && prop.startsWith("lp_")) ||
				(auxCon != null && prop.startsWith("aux_")) ||
				(modemCon != null && prop.startsWith("modem_"))) {
				props.remove(prop);
			}
		}
	}

	private void doCfgEnable(JComboBox cb, JButton bt) {
		String s = (String)cb.getSelectedItem();
		bt.setEnabled(s.startsWith("Diablo630"));
	}

	class CfgTypes extends JComboBox<String> {
		public JButton cfg = null;
		public CfgTypes(String[] tys) {
			super(tys);
		}
	}

	class CfgButton extends JButton {
		public JComboBox type;
		public JTextField text;
		public CfgButton(CfgTypes typ, JTextField txt, ActionListener lstn) {
			super(cfgBtn);
			typ.cfg = this;
			setDisabledIcon(cfgOffBtn);
			addActionListener(lstn);
			type = typ;
			text = txt;
		}
	}

	private void portPlug(Serial15 pt) {
		// TODO: pop-up dialog for attachment.
		JPanel pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		JLabel lb = new JLabel(pt.getName() + ": ");
		CfgTypes types = new CfgTypes(srvs);
		types.addActionListener(this);
		JTextField tf = new JTextField();
		tf.setPreferredSize(new Dimension(200, 20));
		JButton cfg = new CfgButton(types, tf, this);
		doCfgEnable(types, cfg);
		String v = pt.getValue();
		if (v != null) {
			String[] s;
			if (!Character.isLetterOrDigit(v.charAt(0))) {
				s = new String[]{ "" + v.charAt(0), v.substring(1) };
			} else {
				s = v.split("\\s", 2);
			}
			for (String t : srvs) {
				if (t.equals(s[0])) {
					types.setSelectedItem(t);
					break;
				}
			}
			if (s.length > 1) {
				tf.setText(s[1]);
			}
		}
		pn.add(lb);
		pn.add(types);
		pn.add(tf);
		pn.add(cfg);
		JOptionPane jop = new JOptionPane(pn, JOptionPane.PLAIN_MESSAGE,
			JOptionPane.OK_CANCEL_OPTION);
		JDialog dlg = jop.createDialog(top, "Configure " + pt.getName());
		dlg.setVisible(true);
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return;
		}
		if (lstn != null) {
			lstn.propertyChanged();
		}
		v = (String)types.getSelectedItem(); // never length 0
		if (v.equals(srvs[0])) {
			pt.setValue(null);
			return;
		}
		if (v.length() > 1 ||
				Character.isLetterOrDigit(v.charAt(0))) {
			v += " ";
		}
		v += tf.getText();
		pt.setValue(v);
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof Serial15) {
			portPlug((Serial15)e.getSource());
		} else if (e.getSource() instanceof CfgTypes) {
			CfgTypes types = (CfgTypes)e.getSource();
			doCfgEnable(types, types.cfg);
		} else if (e.getSource() instanceof CfgButton) {
			CfgButton cfg = (CfgButton)e.getSource();
			String type = (String)cfg.type.getSelectedItem();
			if (type.startsWith("Diablo630")) {
				Diablo630Dialog targ = new Diablo630Dialog(cfg.text.getText());
				if (!targ.configure(top, 0)) {
					return;
				}
				cfg.text.setText(targ.getArgs(0));
			}
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}
}
