// Copyright (c) 2016 Douglas Miller

import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class SocketDialog extends JPanel implements NetworkNode {
	private JTextField hstField;
	private JTextField prtField;
	private String host = null;
	private int port = -1;
	String[] argv = null; // one-shot after ctor only...

	public SocketDialog(String args) {
		super();
		JPanel pn;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		if (args != null) {
			argv = args.split("\\s");
		}

		hstField = new JTextField();
		hstField.setPreferredSize(new Dimension(200, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Hostname"));
		pn.add(hstField);
		add(pn);

		prtField = new JTextField();
		prtField.setPreferredSize(new Dimension(50, 20));
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Host Port"));
		pn.add(prtField);
		add(pn);
	}

	public boolean isType(String type) {
		return type.equalsIgnoreCase("Socket");
	}

	// cpnetserver%02x_host
	// cpnetserver%02x_port
	public void setProperties(Properties props, int node) {
		String s;
		String pfx = String.format("cpnetserver%02x_", node);
		host = null;
		s = props.getProperty(pfx + "host");
		if (s != null) {
			host = s;
		}
		port = -1;
		s = props.getProperty(pfx + "port");
		if (s != null) {
			try {
				port = Integer.valueOf(s);
			} catch (Exception ee) {}
		}
		if (argv != null) {
			// argv[0] is "Socket"...
			// TODO: syntax check?
			if (argv.length > 1) {
				if (!argv[1].isEmpty()) {
					host = argv[1];
					hstField.setText(argv[1]);
				}
				if (argv.length > 2 && !argv[2].isEmpty()) {
					prtField.setText(argv[2]);
					try {
						port = Integer.valueOf(argv[2]);
					} catch (Exception ee) {}
				}
			}
			argv = null;
		}
	}

	public void updateProperties(Properties props, int node) {
		String pfx = String.format("cpnetserver%02x_", node);
		// Must remove all existing printers and rebuild list.
		removeProperties(props, node);
		if (host != null) {
			props.setProperty(pfx + "host", host);
		}
		if (port > 0) {
			props.setProperty(pfx + "port", Integer.toString(port));
		}
	}

	public void removeProperties(Properties props, int node) {
		String pfx = String.format("cpnetserver%02x_", node);
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith(pfx)) {
				props.remove(prop);
			}
		}
	}

	public boolean configure(Component top, int node) {
		JOptionPane jop = new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
						JOptionPane.OK_CANCEL_OPTION);
		JDialog dlg = jop.createDialog(top,
			String.format("Configure Socket Node %02x", node));
		dlg.setVisible(true);
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return false;
		}
		String s = hstField.getText();
		if (!s.isEmpty()) {
			host = s;
		}
		s = prtField.getText();
		try {
			port = Integer.valueOf(s);
		} catch (Exception ee) {}
		return true;
	}
}
