// Copyright (c) 2016 Douglas Miller

import java.util.Properties;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class V89Dialog extends JPanel implements ActionListener {
	private JComboBox<String> disas;
	private JTextField nameplate;
	private JTextField log;
	private JTextField paste_rate;
	private JTextField paste_cr;
	private Vector<JTextField> sufxField;
	private Vector<JTextField> sufdField;
	private JButton addb;
	private GridBagLayout gb;
	private GridBagConstraints gc;
	JOptionPane jop;
	JDialog dlg;

	public V89Dialog(Properties props) {
		super();
		String s;
		JPanel pn;
		gb = new GridBagLayout();
		gc = new GridBagConstraints();
		gc.fill = GridBagConstraints.NONE;
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 1;
		gc.weighty = 1;
		gc.gridwidth = 1;
		gc.gridheight = 1;
		gc.insets.left = 2;
		gc.insets.right = 2;
		gc.insets.top = 2;
		gc.insets.bottom = 2;
		gc.anchor = GridBagConstraints.CENTER;

		setLayout(gb);
		setOpaque(true);

		addb = new JButton("+");
		addb.addActionListener(this);

		String[] disl = new String[]{
			"",
			"Zilog",
			"MAC80",
		};
		disas = new JComboBox<String>(disl);
		s = props.getProperty("h89_disas");
		for (String f : disl) {
			if (f.equals(s)) {
				disas.setSelectedItem(f);
				break;
			}
		}

		s = props.getProperty("h89_nameplate");
		nameplate = new JTextField();
		nameplate.setPreferredSize(new Dimension(200, 20));
		if (s != null) {
			nameplate.setText(s);
		}

		s = props.getProperty("h89_log");
		log = new JTextField();
		log.setPreferredSize(new Dimension(100, 20));
		if (s != null) {
			log.setText(s);
		}

		s = props.getProperty("h19_paste_rate");
		paste_rate = new JTextField();
		paste_rate.setPreferredSize(new Dimension(50, 20));
		if (s != null) {
			paste_rate.setText(s);
		}

		s = props.getProperty("h19_paste_cr_wait");
		paste_cr = new JTextField();
		paste_cr.setPreferredSize(new Dimension(50, 20));
		if (s != null) {
			paste_cr.setText(s);
		}

		gc.gridwidth = 2;
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("System Log"));
		pn.add(log);
		gb.setConstraints(pn, gc);
		add(pn);
		++gc.gridy;

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Nameplate"));
		pn.add(nameplate);
		gb.setConstraints(pn, gc);
		add(pn);
		++gc.gridy;

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Z80 Disassembly"));
		pn.add(disas);
		gb.setConstraints(pn, gc);
		add(pn);
		++gc.gridy;

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Paste Rate"));
		pn.add(paste_rate);
		gb.setConstraints(pn, gc);
		add(pn);
		++gc.gridy;

		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.X_AXIS));
		pn.add(new JLabel("Paste CR Wait"));
		pn.add(paste_cr);
		gb.setConstraints(pn, gc);
		add(pn);
		++gc.gridy;

		JSeparator sp = new JSeparator();
		// not needed if layout mgr stretches objects...
		sp.setPreferredSize(new Dimension(300, 3));
		gb.setConstraints(sp, gc);
		add(sp);
		++gc.gridy;

		JLabel lb = new JLabel("Floppy Disk Image Suffixes:");
		gb.setConstraints(lb, gc);
		add(lb);
		++gc.gridy;
		gc.gridwidth = 1;

		addHeadings("Suffix", "Description");
		sufxField = new Vector<JTextField>();
		sufdField = new Vector<JTextField>();
		s = props.getProperty("h89_image_sufx");
		String t = props.getProperty("h89_image_sufd");
		if (s != null && t != null) {
			String[] ss = s.split("[ \t]");
			String[] tt = t.split("[ \t]");
			if (ss.length == tt.length && ss.length > 0) {
				for (int x = 0; x < ss.length; ++x) {
					addSufx(ss[x], tt[x].replaceAll("_"," "));
				}
			}
		}
		gb.setConstraints(addb, gc);
		add(addb);

		jop = new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION);
	}

	private void addHeadings(String sfx, String sfd) {
		JLabel lb = new JLabel(sfx);
		gb.setConstraints(lb, gc);
		add(lb);
		++gc.gridx;
		lb = new JLabel(sfd);
		gb.setConstraints(lb, gc);
		add(lb);
		++gc.gridx;
		gc.gridx = 0;
		++gc.gridy;
	}

	private void addSufx(String sfx, String sfd) {
		JTextField tf1 = new JTextField();
		tf1.setPreferredSize(new Dimension(100, 20));
		if (sfx != null) {
			tf1.setText(sfx);
		}
		JTextField tf2 = new JTextField();
		tf2.setPreferredSize(new Dimension(200, 20));
		if (sfd != null) {
			tf2.setText(sfd);
		}
		sufxField.add(tf1);
		sufdField.add(tf2);
		gb.setConstraints(tf1, gc);
		add(tf1);
		++gc.gridx;
		gb.setConstraints(tf2, gc);
		add(tf2);
		++gc.gridx;
		gc.gridx = 0;
		++gc.gridy;
	}

	public boolean doDialog(Component top, String title) {
		dlg = jop.createDialog(top, title);
		dlg.setVisible(true);
		Object res = jop.getValue();
		if (res == null || res.equals(JOptionPane.CANCEL_OPTION)) {
			return false;
		}
		return true;
	}

	public void updateProperties(Properties props) {
		String s = (String)disas.getSelectedItem();
		if (s.isEmpty()) {
			props.remove("h89_disas");
		} else {
			props.setProperty("h89_disas", s);
		}
		s = log.getText();
		if (s.isEmpty()) {
			props.remove("h89_log");
		} else {
			props.setProperty("h89_log", s);
		}
		s = nameplate.getText();
		if (s.isEmpty()) {
			props.remove("h89_nameplate");
		} else {
			props.setProperty("h89_nameplate", s);
		}
		s = paste_rate.getText();
		if (s.isEmpty()) {
			props.remove("h19_paste_rate");
		} else {
			props.setProperty("h19_paste_rate", s);
		}
		s = paste_cr.getText();
		if (s.isEmpty()) {
			props.remove("h19_paste_cr_wait");
		} else {
			props.setProperty("h19_paste_cr_wait", s);
		}
		String ss = "";
		String tt = "";
		if (sufxField.size() > 0) {
			String t;
			for (int x = 0; x < sufxField.size(); ++x) {
				s = sufxField.get(x).getText();
				t = sufdField.get(x).getText().replaceAll(" ","_");
				if (s.isEmpty() || t.isEmpty()) {
					continue;
				}
				if (!ss.isEmpty()) {
					ss += " ";
					tt += " ";
				}
				ss += s;
				tt += t;
			}
		}
		if (!ss.isEmpty()) {
			props.setProperty("h89_image_sufx", ss);
			props.setProperty("h89_image_sufd", tt);
		} else {
			props.remove("h89_image_sufx");
			props.remove("h89_image_sufd");
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (!(e.getSource() instanceof JButton)) {
			return;
		}
		JButton bp = (JButton)e.getSource();
		if (bp == addb) {
			remove(addb);
			addSufx(null, null);
			gb.setConstraints(addb, gc);
			add(addb);
			dlg.validate();
			dlg.pack();
			dlg.repaint();
			return;
		}
	}
}
