// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Map;
import java.util.HashMap;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Properties;

public class Z17 extends AddOnCard
		implements ChangeListener, WindowListener, ActionListener {

	private IDC34 p803;
	private Map<Integer, String> drives5;
	private Map<Integer, DiskImageFile> disks5;
	private ChangeListener lstn;
	private String[] types5 = new String[]{
		"FDD_5_25_SS_ST",
		"FDD_5_25_DS_ST",
		"FDD_5_25_SS_DT",
		"FDD_5_25_DS_DT",
	};

	public Z17(ChangeListener lstn) {
		super("Configure Z17");
		this.lstn = lstn;

		drives5 = new HashMap<Integer, String>();
		disks5 = new HashMap<Integer, DiskImageFile>();

		addWindowListener(this);

		layoutCard();
		connectorListener(this);

		////////////////////////////////////////////////
		// Setup menus
		//
		JMenuBar mb = new JMenuBar();
		JMenu mu;
		JMenuItem mi;
		mu = new JMenu("Card");
		mi = new JMenuItem("Remove", KeyEvent.VK_R);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Done", KeyEvent.VK_D);
		mi.addActionListener(this);
		mu.add(mi);
		// TODO: can we cancel (cleanly) here?
		mb.add(mu);
		setJMenuBar(mb);

		////////////////////////////////////////////////
		// Layout Z17 PCB...
		//
		JPanel pn;

		gc.anchor = GridBagConstraints.NORTH;
		p803 = new IDC34("P803", "z17_drive");
		p803.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p803);
		pn.add(new JLabel("5\" FLOPPY"));
		place(pn, 2, 0, 2, 1);

		pn = new JPanel();
		pn.setPreferredSize(new Dimension(800,300));
		pn.setOpaque(false);
		pn.add(new JLabel("Heath H17 Hard-Sector Floppy Controller"));
		place(pn, 1, 2, 5, 2);

		finish();
	}

	public void setProperties(Properties props) {
		String s;
		// TODO: handle z17_dir...?
		drives5.clear();
		disks5.clear();
		for (int x = 0; x < 4; ++x) {
			s = String.format("z17_drive%d", x + 1);
			s = props.getProperty(s);
			if (s != null) {
				drives5.put(x, s);
			}
			s = String.format("z17_disk%d", x + 1);
			s = props.getProperty(s);
			if (s != null) {
				disks5.put(x, new DiskImageFile(s.split("\\s")));
			}
		}
		p803.setValue(drives5.size());
	}

	public void updateProperties(Properties props) {
		String s;
		for (Integer drv : drives5.keySet()) {
			s = String.format("z17_drive%d", drv + 1);
			props.setProperty(s, drives5.get(drv));
		}
		for (Integer drv : disks5.keySet()) {
			s = String.format("z17_disk%d", drv + 1);
			props.setProperty(s, disks5.get(drv).getProp());
		}
	}

	public void removeProperties(Properties props) {
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("z17_")) {
				props.remove(prop);
			}
		}
	}

	public String getCardName() { return "Z17"; }

	public boolean configure(Component top) {
		setVisible(true);
		return true;
	}

	public void windowActivated(WindowEvent e) { }
	public void windowClosed(WindowEvent e) { }
	public void windowIconified(WindowEvent e) { }
	public void windowOpened(WindowEvent e) { }
	public void windowDeiconified(WindowEvent e) { }
	public void windowDeactivated(WindowEvent e) { }
	public void windowClosing(WindowEvent e) {
		setVisible(false);
	}

	private void drive5Plug(IDC34 cs) {
		DiskDialog dd = new DiskDialog(1, 3, types5, null, 0,
			"Num", "Drive Type", "Label", "Media", drives5, disks5);
		if (!dd.doDialog(this, "Configure Z17 Drives")) {
			return;
		}
		drives5 = dd.getDriveMap();
		disks5 = dd.getMediaMap();
		p803.setValue(drives5.size());
		propertyChanged();
	}

	private void menuClick(JMenuItem mi) {
		if (mi.getMnemonic() == KeyEvent.VK_R) {
			setVisible(false);
			if (lstn != null) {
				lstn.cardRemoved(this);
			}
		} else if (mi.getMnemonic() == KeyEvent.VK_D) {
			setVisible(false);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof IDC34) {
			drive5Plug((IDC34)e.getSource());
		} else if (connectorPerformed(e)) {
			setVisible(false);
		} else if (e.getSource() instanceof JMenuItem) {
			menuClick((JMenuItem)e.getSource());
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}

	public void cardRemoved(AddOnCard cd) {} // never called from widgets

	public void propertyChanged() {
		// TODO: what else?
		if (lstn != null) {
			lstn.propertyChanged();
		}
	}
}
