// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Map;
import java.util.HashMap;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Properties;

public class Z37 extends AddOnCard
		implements ChangeListener, WindowListener, ActionListener {

	private IDC34 p3;
	private IDC34 p4;
	private Jumper3x1 j1;
	private Jumper3x1 j2;
	private Map<Integer, String> drives5;
	private Map<Integer, DiskImageFile> disks5;
	private ChangeListener lstn;
	private String[] types5 = new String[]{
		"FDD_5_25_SS_ST",
		"FDD_5_25_DS_ST",
		"FDD_5_25_SS_DT",
		"FDD_5_25_DS_DT",
	};

	public Z37(ChangeListener lstn) {
		super("Configure Z37");
		this.lstn = lstn;

		drives5 = new HashMap<Integer, String>();
		disks5 = new HashMap<Integer, DiskImageFile>();

		addWindowListener(this);

		layoutCard();
		connectorListener(this);

		////////////////////////////////////////////////
		// Setup menus
		//
		JMenuBar mb = new JMenuBar();
		JMenu mu;
		JMenuItem mi;
		mu = new JMenu("Card");
		mi = new JMenuItem("Remove", KeyEvent.VK_R);
		mi.addActionListener(this);
		mu.add(mi);
		mi = new JMenuItem("Done", KeyEvent.VK_D);
		mi.addActionListener(this);
		mu.add(mi);
		// TODO: can we cancel (cleanly) here?
		mb.add(mu);
		setJMenuBar(mb);

		////////////////////////////////////////////////
		// Layout Z37 PCB...
		//
		JPanel pn;

		gc.anchor = GridBagConstraints.NORTH;

		p3 = new IDC34("P3", "z37_drive");
		p3.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p3);
		pn.add(new JLabel("INTERNAL FLOPPY"));
		place(pn, 2, 0, 2, 1);

		p4 = new IDC34("P4", "z37_drive");
		p4.addActionListener(this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(p4);
		pn.add(new JLabel("EXTERNAL FLOPPY"));
		place(pn, 4, 0, 2, 1);

		gc.gridwidth = 3;
		gc.gridheight = 2;
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(500, 300));
		pn.setOpaque(false);
		pn.add(new JLabel("Zenith Z37 Soft-Sector Floppy Controller"));
		place(pn, 1, 1, 5, 1);

		j1 = new Jumper3Vert("J1", this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("A"));
		pn.add(j1);
		pn.add(new JLabel("B"));
		JPanel pn2 = new JPanel();
		pn2.setLayout(new BoxLayout(pn2, BoxLayout.X_AXIS));
		pn2.setOpaque(false);
		pn2.add(new JLabel("J1"));
		pn2.add(pn);
		place(pn2, 3, 2, 1, 1);

		j2 = new Jumper3Vert("J2", this);
		pn = new JPanel();
		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.setOpaque(false);
		pn.add(new JLabel("A"));
		pn.add(j2);
		pn.add(new JLabel("B"));
		pn2 = new JPanel();
		pn2.setLayout(new BoxLayout(pn2, BoxLayout.X_AXIS));
		pn2.setOpaque(false);
		pn2.add(new JLabel("J2"));
		pn2.add(pn);
		place(pn2, 4, 2, 1, 1);

		finish();
	}

	public void setProperties(Properties props) {
		String s;
		// TODO: handle z37_dir...?
		int v = 0;
		s = props.getProperty("z37_j1");
		if (s == null) {
			s = props.getProperty("z37_p1");
			props.remove("z37_p1");
		}
		if (s != null && s.equalsIgnoreCase("b")) {
			v |= 1;
		}
		j1.setValue(v);
		v = 0;
		s = props.getProperty("z37_j2");
		if (s == null) {
			s = props.getProperty("z37_p2");
			props.remove("z37_p2");
		}
		if (s != null && s.equalsIgnoreCase("b")) {
			v |= 1;
		}
		j2.setValue(v);
		drives5.clear();
		disks5.clear();
		for (int x = 0; x < 4; ++x) {
			s = String.format("z37_drive%d", x + 1);
			s = props.getProperty(s);
			if (s != null) {
				drives5.put(x, s);
			}
			s = String.format("z37_disk%d", x + 1);
			s = props.getProperty(s);
			if (s != null) {
				disks5.put(x, new DiskImageFile(s.split("\\s")));
			}
		}
		p3.setValue(drives5.size());
		p4.setValue(drives5.size());
	}

	public void updateProperties(Properties props) {
		String s;
		s = "A";
		if (j1.getValue() != 0) {
			s = "B";
		}
		props.setProperty("z37_j1", s);
		s = "A";
		if (j2.getValue() != 0) {
			s = "B";
		}
		props.setProperty("z37_j2", s);
		for (Integer drv : drives5.keySet()) {
			s = String.format("z37_drive%d", drv + 1);
			props.setProperty(s, drives5.get(drv));
		}
		for (Integer drv : disks5.keySet()) {
			s = String.format("z37_disk%d", drv + 1);
			props.setProperty(s, disks5.get(drv).getProp());
		}
	}

	public void removeProperties(Properties props) {
		for (String prop : props.stringPropertyNames()) {
			if (prop.startsWith("z37_")) {
				props.remove(prop);
			}
		}
	}

	public String getCardName() { return "Z37"; }

	public boolean configure(Component top) {
		setVisible(true);
		return true;
	}

	public void windowActivated(WindowEvent e) { }
	public void windowClosed(WindowEvent e) { }
	public void windowIconified(WindowEvent e) { }
	public void windowOpened(WindowEvent e) { }
	public void windowDeiconified(WindowEvent e) { }
	public void windowDeactivated(WindowEvent e) { }
	public void windowClosing(WindowEvent e) {
		setVisible(false);
	}

	private void drive5Plug(IDC34 cs) {
		DiskDialog dd = new DiskDialog(1, 4, types5, null, 0,
			"Num", "Drive Type", "Label", "Media", drives5, disks5);
		if (!dd.doDialog(this, "Configure Z37 Drives")) {
			return;
		}
		drives5 = dd.getDriveMap();
		disks5 = dd.getMediaMap();
		p3.setValue(drives5.size());
		p4.setValue(drives5.size());
		propertyChanged();
	}

	private void menuClick(JMenuItem mi) {
		if (mi.getMnemonic() == KeyEvent.VK_R) {
			setVisible(false);
			if (lstn != null) {
				lstn.cardRemoved(this);
			}
		} else if (mi.getMnemonic() == KeyEvent.VK_D) {
			setVisible(false);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof IDC34) {
			drive5Plug((IDC34)e.getSource());
		} else if (connectorPerformed(e)) {
			setVisible(false);
		} else if (e.getSource() instanceof JMenuItem) {
			menuClick((JMenuItem)e.getSource());
		} else {
			System.err.format("Unknown action event %s\n", e.getSource().getClass().getName());
		}
	}

	public void cardRemoved(AddOnCard cd) {} // never called from widgets

	public void propertyChanged() {
		// TODO: what else?
		if (lstn != null) {
			lstn.propertyChanged();
		}
	}
}
